Package: JM4QTN
Type: Package
Title: Joint Mapping for Quantitative Trait Loci
Version: 1.0.0
Date: 2026-05-09
Authors@R: c(person(given = "Junhui", family = "Li",
           role = c("aut", "cre"),
           email = "ljh.biostat@gmail.com"),
           person(given = "Wenxin", family = "Liu",
           role = "aut"))
Description: A comprehensive computational framework for joint mapping, developed by Li (2016) <doi:10.11841/j.issn.1007-4333.2016.06.002>, supports quantitative trait locus detection in structured genetic populations. It integrates robust phenotype summarization, computes genotype probabilities, and imputes missing markers for association and linkage mapping. Empirical significance thresholds are estimated via permutation testing coupled with stepwise regression. The framework enables genome-wide scans under both univariate and multivariate trait models, streamlining the discovery of complex genetic architectures.
License: GPL (>= 2)
Imports: lsmeans (>= 2.25.5), StepReg (>= 1.5.0)
Depends: R (>= 3.5.0)
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-05-13 23:22:10 UTC; lij11
Author: Junhui Li [aut, cre],
  Wenxin Liu [aut]
Maintainer: Junhui Li <ljh.biostat@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-19 08:10:02 UTC
Built: R 4.5.2; ; 2026-05-19 09:59:55 UTC; unix

# Generated by roxygen2: do not edit by hand

export(expected_genotype_dist)
export(genotype_freq)
export(genotype_prob)
export(haldane_map)
export(joint_map)
export(permutation_test)
export(pheno_stats)
export(skeletion_build)
export(skeleton_build)
importFrom(StepReg,stepwise)
importFrom(stats,anova)
importFrom(stats,as.formula)
importFrom(stats,df.residual)
importFrom(stats,lm)
importFrom(stats,predict)
importFrom(stats,resid)
importFrom(stats,shapiro.test)
importFrom(stats,terms)
importFrom(stats,update)

# JM4QTN: Joint Mapping for Quantitative Trait Loci

[![R-CMD-check](https://github.com/JunhuiLi1017/JM4QTN/workflows/R-CMD-check/badge.svg)](https://github.com/JunhuiLi1017/JM4QTN/actions)
[![License: GPL v2](https://img.shields.io/badge/License-GPL%20v2-blue.svg)](https://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html)

## Overview

**JM4QTN** supports joint mapping and related analyses for quantitative trait loci (QTL). The package provides phenotype summaries, genotype probability and imputation (association vs linkage mapping), permutation-based thresholds with stepwise regression (via **StepReg**), and scan statistics comparing full vs reduced linear models (**joint_map**).

## Key Features

### Core analysis

- **Permutation testing** (`permutation_test`) for empirical p-value and LOD cutoffs under stepwise selection  
- **Skeleton model** (`skeleton_build` / `skeletion_build`) fitting the selected model using those cutoffs  
- **Joint mapping scan** (`joint_map`) over candidate terms vs the skeleton model  

### Genotypes and map

- **Genotype probabilities / imputation** (`genotype_prob`) for AM or LM, optional virtual markers  
- **Expected genotype distribution** (`expected_genotype_dist`) and **genotype frequencies** (`genotype_freq`)  
- **Haldane map** (`haldane_map`) for recombination  

### Phenotypes

- **Phenotype statistics** (`pheno_stats`): normality, ANOVA, least squares means (via **lsmeans**)

## Installation

```r
# Install from GitHub
if (!require("devtools")) install.packages("devtools")
devtools::install_github("JunhuiLi1017/JM4QTN")

## or install from CRAN
install.pakcages("JM4QTN")

# Load the package
library(JM4QTN)
```

## Quick start

### 1. Phenotype analysis (`pheno_stats`)

Designed for multi-environment trials: columns **E**, **B**, **R**, **G**, then traits.

```r
pheno_data <- data.frame(
  E = rep(c("Env1", "Env2"), each = 60),
  B = rep(c("B1", "B2"), each = 30, times = 2),
  R = rep(1:5, 24),
  G = factor(rep(1:12, 10)),
  Height = rnorm(120, 175, 8),
  Weight = rnorm(120, 75, 12)
)

results <- pheno_stats(pheno_data)
results$Height$normality_test
results$Height$ANOVA
results$Height$lsmeans
```

### 2. Genotype probabilities (`genotype_prob`)

```r
genetic_map <- data.frame(
  marker = c("M1", "M2", "M3"),
  chr = c(1, 1, 1),
  pos = c(0, 10, 20)
)
geno_data <- matrix(sample(0:2, 15, TRUE), nrow = 5, ncol = 3,
                    dimnames = list(paste0("Ind", 1:5), c("M1", "M2", "M3")))

result_am <- genotype_prob(genetic_map, geno_data, method = "AM")
result_lm <- genotype_prob(genetic_map, geno_data, method = "LM",
                           croType = "F2", steps = 0)
```

### 3. Permutation thresholds, skeleton model, and joint mapping

Formula-based workflow with genotype columns bound into `data` (see package examples for `joint_map`).

```r
pheno_data <- data.frame(
  Trait1 = rnorm(100, 100, 15),
  Trait2 = rnorm(100, 50, 8),
  Popu = rep(c("Pop1", "Pop2"), each = 50)
)
geno_data <- matrix(sample(0:2, 100 * 50, TRUE), nrow = 100, ncol = 50)
colnames(geno_data) <- paste0("M", 1:50)
data1 <- cbind(pheno_data, geno_data)

terms <- c("Popu", paste0(colnames(geno_data), ":Popu"))
formula1 <- reformulate(terms, response = "Trait1")

cut_off_list <- permutation_test(formula1, data1, n = 100, alpha = 0.1)

skeleton <- skeleton_build(
  formula1, data1,
  strategy = "bidirection", metric = "SL",
  cut_off_list = cut_off_list
)

results <- joint_map(
  formula1, data1, skeleton,
  include = "Popu", cut_off_list = cut_off_list
)
results$p_value
results$lod
```

### 4. Supporting utilities

```r
haldane_map(0.1)   # genetic distance (Morgan) -> recombination fraction
genotype_freq("Fn", generation = 3, genotype_index = 1, recomb_aq = 0.1, recomb_qb = 0.2)
expected_genotype_dist("22", "F2", Gn = 2, x = 0.1, y = 0.2)
```

## Function reference (exported)

| Function | Description |
|----------|-------------|
| `joint_map()` | Compare candidate model terms to a fitted skeleton; returns p-values and LOD-like statistics |
| `permutation_test()` | Permutation distribution and empirical cutoffs for stepwise linear models |
| `skeleton_build()` | Alias for `skeletion_build()` (preferred spelling) |
| `skeletion_build()` | Fit stepwise skeleton using permutation p-value as StepReg entry/stay levels |
| `genotype_prob()` | Genotype probabilities / imputation (AM or LM) |
| `genotype_freq()` | Genotype class frequencies by cross type and generation |
| `expected_genotype_dist()` | Expected genotype distribution by marker pattern and cross |
| `haldane_map()` | Haldane mapping function |
| `pheno_stats()` | Normality tests, ANOVA, lsmeans for structured phenotype tables |

**Backward-compatible names** (same package, alternate spelling): `calculate_genotype_frequencies`, `calculate_expected_genotype_distribution`, `haldane_mapping_function` — see individual help pages.

## Data notes

- **`pheno_stats`**: expects **E**, **B**, **R**, **G** as the first four columns, then traits.  
- **`genotype_prob`**: rows = individuals, columns = markers; column order must align with `GeneticMap` rows.  
- **`joint_map` / `permutation_test`**: supply a single `data` frame containing phenotypes, population, and numeric genotype columns used in the formula.

## Citation

```bibtex
@Manual{JM4QTN,
  title = {JM4QTN: Joint Mapping for Quantitative Trait Loci},
  author = {Junhui Li and Wenxin Liu},
  year = {2026},
  note = {R package version 1.0.0},
  url = {https://github.com/JunhuiLi1017/JM4QTN}
}
```

## References

- Jiang, C. and Zeng, Z.B. (1995). Multiple trait analysis of genetic mapping for quantitative trait loci. *Genetics*, 140(3), 1111-1127.
- Churchill, G.A. and Doerge, R.W. (1994). Empirical threshold values for quantitative trait mapping. *Genetics*, 138(3), 963-971.
- Haldane, J.B.S. (1919). The combination of linkage values and the calculation of distances between the loci of linked factors. *Journal of Genetics*, 8(3), 299-309.

## Contributing

Issues and pull requests are welcome on [GitHub](https://github.com/JunhuiLi1017/JM4QTN).

## License

This project is licensed under the GNU General Public License v2.0 - see the [GNU GPLv2 license text](https://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html) for details.

## Contact

- **Maintainer**: Junhui Li <junhuili@cau.edu.cn>
- **GitHub**: [https://github.com/JunhuiLi1017/JM4QTN](https://github.com/JunhuiLi1017/JM4QTN)

#' Permutation Test for Stepwise Regression
#' 
#' Performs permutation tests for stepwise regression to determine empirical significance thresholds
#' for QTL detection using stepwise regression. This function implements a comprehensive permutation
#' testing framework for stepwise regression.
#' 
#' @param formula A model formula defining the response and candidate predictors for
#'   permutation testing.
#' @param data A data frame containing all variables referenced in \code{formula},
#'   including response trait(s) and predictor variables.
#' @param n Integer number of permutations to run. Larger values provide more stable
#'   empirical thresholds but increase computation time. Default is \code{1000}.
#' @param alpha Numeric significance level used to extract empirical cutoff values
#'   from the permutation distributions. Must be between 0 and 1. Default is \code{0.1}.
#' @param type Type of regression model.
#'   Typical values include \code{"linear"}, \code{"logistic"}, and \code{"poisson"}.
#'   Default is \code{"linear"}.
#' @param include Optional character vector of variable names that should always be
#'   included during stepwise model selection. If \code{NULL}, inclusion terms can be
#'   inferred from interaction terms in \code{formula}. Default is \code{NULL}.
#' @param strategy Stepwise selection strategy passed to \code{StepReg::stepwise()}.
#'   Typical values include \code{"forward"}, \code{"backward"}, and \code{"bidirection"}.
#'   Default is \code{"bidirection"}.
#' @param metric Model selection metric used inside stepwise regression.
#'   Typical values include \code{"AIC"}, \code{"BIC"}, or \code{"SBC"}.
#'   Default is \code{"SBC"}.
#' @param verbose Logical; if \code{TRUE}, emit progress \code{message()}s during
#'   permutations. Default is \code{FALSE}.
#' @return A list containing the empirical significance thresholds for p-value and LOD score with:
#'   \itemize{
#'     \item \strong{cut_off}: A vector containing the empirical significance thresholds for p-value and LOD score
#'     \item \strong{pvalue}: A vector containing the p-values for each permutation
#'     \item \strong{lod}: A vector containing the LOD scores for each permutation
#'   }
#' 
#' @details This function implements a comprehensive permutation testing framework.
#' 
#' \strong{Output Interpretation:}
#' \itemize{
#'   \item \strong{P-values}: Empirical significance thresholds for p-value
#'   \item \strong{LOD scores}: Empirical significance thresholds for LOD score
#' }
#' 
#' @examples
#' \donttest{
#' # Example phenotype data
#' set.seed(1)
#' pheno_data <- data.frame(
#'   Trait1 = rnorm(100, mean = 100, sd = 15),
#'   Trait2 = rnorm(100, mean = 50, sd = 8),
#'   Popu = rep(c("Pop1", "Pop2"), each = 50)
#' )
#' 
#' # Example genotype data
#' geno_data <- matrix(sample(c(0,1,2), 100*50, replace = TRUE), 
#'                     nrow = 100, ncol = 50)
#' colnames(geno_data) <- paste0("M", 1:50)
#' 
#' data1 <- cbind(pheno_data,geno_data)
#' 
#' data1$Popu <- as.factor(data1$Popu)
#' 
#' terms <- c("Popu", paste0(colnames(geno_data), ":Popu"))
#' formula1 <- reformulate(terms, response = "Trait1")
#' 
#' cut_off_list <- permutation_test(formula1, data1, n = 10, alpha = 0.1)
#'
#' formula2 <- reformulate(terms, response = "cbind(Trait1,Trait2)")
#'
#' cut_off_list <- permutation_test(formula2, data1, n = 10, alpha = 0.1)
#' }
#' 
#' @importFrom stats resid anova update terms lm df.residual
#' 
#' @export
permutation_test <- function(formula, data, n = 1000, alpha = 0.1, include = NULL,
                             strategy = "bidirection", metric = "SBC", type = "linear",
                             verbose = FALSE) {
  
  ## get dependent variable name
  term_form <- terms(formula, data = data)
  vars <- as.character(attr(term_form, "variables"))[ -1 ]
  y_name <- vars[attr(term_form, "response")]
  
  if (startsWith(y_name, "cbind(")) {
    inner_content <- gsub("^cbind\\(|\\)$", "", y_name)
    y_var <- trimws(strsplit(inner_content, ",")[[1]])
  } else {
    y_var <- y_name
  }
  ## get independent variable name
  inlude_var <- include
  x_name <- attr(term_form, "term.labels")
  continuous_check <- grepl(":", x_name)
  if(any(continuous_check)) {
    cont_class_var <- x_name[continuous_check]
    front_var <- gsub(".*:", "", cont_class_var)
    back_var <- gsub(":.*", "", cont_class_var)
    inlude_var <- unique(back_var)
    data[,inlude_var] <- as.factor(data[,inlude_var])
    x_name_exclude <- x_name[!x_name %in% inlude_var]
  }else{
    x_name_exclude <- x_name
  }

  p_value_pt <- 1:n
  lod_pt <- p_value_pt
  data_pt <- data
  nobs <- nrow(data)
  for(v in 1:n){
    data_pt[1:nobs,y_var] <- data[sample(1:nobs,nobs,replace=FALSE),y_var]
    
    stepwise_var <- StepReg::stepwise(
      formula, data_pt, type = "linear", strategy = strategy, metric = metric,
      include = inlude_var
    )
    
    call_best <- stepwise_var[[strategy]][[metric]]
    formula_best <- call_best$call$formula
    
    term_form_best <- terms(call_best, data = data_pt)
    x_name_best <- attr(term_form_best, "term.labels")
    resd_best <- resid(call_best)
    mat_rss_best <- t(resd_best) %*% resd_best
    det_rss_best <- det(mat_rss_best) 
    df_best <- df.residual(call_best)
    
    p_value <- seq_along(x_name_exclude)
    lod <- p_value

    for(i in seq_along(x_name_exclude)){
      x <- x_name_exclude[i]
      if(x %in% x_name_best){
        formula_simple <- update(formula_best, as.formula(paste(". ~ . -", x)))
        call_simple <- lm(formula_simple,data=data_pt)
        resd_simple <- resid(call_simple)
        mat_rss_simple <- t(resd_simple) %*% resd_simple
        det_rss_simple <- det(mat_rss_simple)
        p_value[i] <- anova(call_best,call_simple)[2,"Pr(>F)"]
        lod[i] <- abs(0.5*nobs*log10(det_rss_best/det_rss_simple))
      }else{
        formula_full <- update(formula_best, as.formula(paste(". ~ . +", x)))
        call_full <- lm(formula_full,data=data_pt)
        resd_full <- resid(call_full)
        mat_rss_full <- t(resd_full) %*% resd_full
        det_rss_full <- det(mat_rss_full)
        df_full <- df.residual(call_full)
        if(df_full == 0){
          p_value[i] <- 1
          lod[i] <- 0
        }else{
          p_value[i] <- anova(call_full,call_best)[2,"Pr(>F)"]
          lod[i] <- abs(0.5*nobs*log10(det_rss_full/det_rss_best))
        }
      }
    }
    p_value_pt[v] <- min(p_value, na.rm = T)
    lod_pt[v] <- max(lod, na.rm = T)
    if (verbose && v %% max(1L, n %/% 10L) == 0L) {
      message(round(100 * v / n), "% of permutations completed at ", Sys.time())
    }
  }
  p_value_sort <- sort(p_value_pt)
  lod_sort <- sort(lod_pt, decreasing = T)
  cut_off <- c(p_value_sort[round(n*alpha)], lod_sort[round(n*alpha)])
  names(cut_off) <- c("pvalue","lod")
  return(list(cut_off = cut_off, pvalue = p_value_sort, lod=lod_sort))
}

#' Joint Mapping Analysis for Multiple Traits
#' 
#' Performs comprehensive joint mapping analysis for multiple traits using either Association Mapping (AM) 
#' or Linkage Mapping (LM) methods to identify QTL affecting multiple traits simultaneously. This function
#' implements sophisticated statistical methods for multivariate QTL analysis with comprehensive
#' cofactor selection and significance testing.
#' 
#' @param formula A model formula defining the full set of candidate linear-model terms
#'   (for example, factor population effects and marker-by-population interactions).
#'   Term labels from this formula are compared to the selected model in \code{skeleton}.
#' @param data A data frame containing all variables used in \code{formula} and in
#'   \code{skeleton$call$formula}.
#' @param skeleton A fitted model object (typically the final model from stepwise
#'   selection) that contains \code{skeleton$call$formula}. Commonly produced by
#'   \code{skeletion_build()} using the same \code{formula}, \code{data},
#'   \code{include}, and \code{cut_off_list}.
#' @param include Optional character vector of predictor names that should be treated
#'   as grouping variables (coerced to factor when \code{formula} contains \code{:}
#'   interactions). If \code{NULL}, interaction-based inclusion may be inferred from
#'   term labels. Matches the \code{include} argument used when fitting \code{skeleton}.
#' @param cut_off_list A list whose component \code{cut_off} includes a named element
#'   \code{"pvalue"} (for example, output from \code{\link{permutation_test}}).
#'   Used upstream with \code{skeletion_build()} to obtain \code{skeleton}; it is
#'   part of the API for a consistent joint-mapping workflow but is not read inside
#'   \code{joint_map()}.
#' 
#' @return A list containing pvalue and LOD for each term in the formula:
#'   \itemize{
#'     \item \code{pvalue}: P-value for each term
#'     \item \code{lod}: LOD score for each term
#'   }
#' 
#' @examples
#' \donttest{
#' # Example phenotype data
#' set.seed(1)
#' pheno_data <- data.frame(
#'   Trait1 = rnorm(100, mean = 100, sd = 15),
#'   Trait2 = rnorm(100, mean = 50, sd = 8),
#'   Popu = rep(c("Pop1", "Pop2"), each = 50)
#' )
#' 
#' # Example genotype data
#' geno_data <- matrix(sample(c(0,1,2), 100*50, replace = TRUE),
#'                     nrow = 100, ncol = 50)
#' colnames(geno_data) <- paste0("M", 1:50)
#' 
#' data1 <- cbind(pheno_data,geno_data) 
#' 
#' data1$Popu <- as.factor(data1$Popu)
#' 
#' terms <- c("Popu", paste0(colnames(geno_data),":Popu"))
#' formula1 <- reformulate(terms, response = "Trait1")
#' 
#' cut_off_list <- permutation_test(formula1, data1, n=10, alpha = 0.1, include="Popu")
#' 
#' skeleton <- skeletion_build(
#'   formula1, data1, strategy = "bidirection", metric = "SBC",
#'   cut_off_list = cut_off_list, include="Popu"
#' )
#' 
#' results <- joint_map(formula1, data1, skeleton, include = "Popu", cut_off_list = cut_off_list)
#' 
#' print(results)
#' 
#'
#' }
#' @seealso \code{\link{permutation_test}} for permutation thresholds,
#'   \code{\link{genotype_prob}} for genotype probability calculations
#' 
#' @references
#' Jiang, C. and Zeng, Z.B. (1995). Multiple trait analysis of genetic mapping for 
#' quantitative trait loci. Genetics, 140(3), 1111-1127.
#' 
#' @importFrom stats anova as.formula df.residual lm resid terms update
#' 
#' @export
#' 

joint_map <- function(formula, data, skeleton, include, cut_off_list){
  nobs <- nrow(data)
  call_best <- skeleton
  formula_best <- skeleton$call$formula
  term_form_best <- terms(formula_best, data = data)
  x_name_best <- attr(term_form_best, "term.labels")

  resd_best <- resid(call_best)
  mat_rss_best <- t(resd_best) %*% resd_best
  det_rss_best <- base::det(mat_rss_best)

  x_name <- attr(terms(formula), "term.labels")
  inlude_var <- include
  continuous_check <- grepl(":", x_name)
  if(any(continuous_check)) {
    cont_class_var <- x_name[continuous_check]
    back_var <- gsub(":.*", "", cont_class_var)
    inlude_var <- unique(back_var)
    data[,inlude_var] <- as.factor(data[,inlude_var])
    x_name_exclude <- x_name[!x_name %in% inlude_var]
  }else{
    x_name_exclude <- x_name
  }
  p_value <- seq_along(x_name_exclude)
  names(p_value) <- x_name_exclude
  lod <- p_value

  for(i in seq_along(x_name_exclude)){
    x <- x_name_exclude[i]
    if(x %in% x_name_best){
      formula_simple <- update(formula_best, as.formula(paste(". ~ . -", x)))
      call_simple <- lm(formula_simple,data=data)
      resd_simple <- resid(call_simple)
      mat_rss_simple <- t(resd_simple) %*% resd_simple
      det_rss_simple <- base::det(mat_rss_simple)
      p_value[i] <- anova(call_simple, call_best)[2, "Pr(>F)"]
      lod[i] <- 0.5 * nobs * log10(det_rss_simple / det_rss_best)
    }else{
      formula_full <- update(formula_best, as.formula(paste(". ~ . +", x)))
      call_full <- lm(formula_full,data=data)
      resd_full <- resid(call_full)
      mat_rss_full <- t(resd_full) %*% resd_full
      det_rss_full <- base::det(mat_rss_full)
      df_full <- df.residual(call_full)
      if(df_full == 0){
        p_value[i] <- 1
        lod[i] <- 0
      }else{
        p_value[i] <- anova(call_best, call_full)[2, "Pr(>F)"]
        lod[i] <- 0.5 * nobs * log10(det_rss_best / det_rss_full)
      }
    }
  }
  return(list(p_value = p_value, lod = lod))
}

#' Expected Allele Genotype Distribution Calculator
#' 
#' Calculates expected allele genotype distribution probabilities for different marker types
#' and cross types in genetic mapping studies. This function implements sophisticated algorithms
#' to compute genotype probabilities based on flanking marker information and population structure.
#' 
#' @param marType Character string specifying the marker type. Options include:
#'   \itemize{
#'     \item \code{"22", "21", "20", "12", "11", "10", "02", "01", "00"}: Both flanking markers observed
#'     \item \code{"N2", "N1", "N0"}: Only right flanking marker observed
#'     \item \code{"2N", "1N", "0N"}: Only left flanking marker observed
#'   }
#'   These codes represent different combinations of flanking marker genotypes.
#' @param croType Character string specifying the cross type:
#'   \itemize{
#'     \item \code{"Fn"}: F-n generation populations
#'     \item \code{"BCP1"}: Backcross to parent 1
#'     \item \code{"BCP2"}: Backcross to parent 2
#'     \item \code{"F2"}: F2 generation
#'     \item \code{"DH"}: Doubled haploid
#'     \item \code{"RIL"}: Recombinant inbred line
#'   }
#' @param Gn Numeric value specifying the generation number. Must be greater than 0.
#' @param x Numeric value representing recombination fraction between loci A and Q.
#'   Must be between 0 and 0.5.
#' @param y Numeric value representing recombination fraction between loci Q and B.
#'   Must be between 0 and 0.5. Default is 0.
#' 
#' @return A numeric value representing the expected probability of the specified genotype.
#'   The result is typically between -1 and 1, where positive values indicate higher
#'   probability of the target genotype.
#' 
#' @details This function calculates expected genotype probabilities using different approaches:
#' 
#' \strong{Marker Type Classification:}
#' \itemize{
#'   \item \strong{Double flanking markers} (\code{"22", "21", "20", etc.}): Both left and right
#'         flanking markers are observed, providing maximum information
#'   \item \strong{Single right flanking marker} (\code{"N2", "N1", "N0"}): Only the right
#'         flanking marker is observed
#'   \item \strong{Single left flanking marker} (\code{"2N", "1N", "0N"}): Only the left
#'         flanking marker is observed
#' }
#' 
#' \strong{Calculation Methods by Cross Type:}
#' \itemize{
#'   \item \strong{Fn populations}: Uses \code{\link{genotype_freq}} for complex calculations
#'         involving multiple genotype classes
#'   \item \strong{F2, DH, RIL}: Uses analytical formulas based on classical genetics
#'   \item \strong{BCP1, BCP2}: Uses \code{\link{genotype_freq}} with specific genotype indices
#' }
#' 
#' \strong{Mathematical Framework:}
#' The function uses different probability calculations depending on the available information:
#' \itemize{
#'   \item When both flanking markers are observed, it uses conditional probabilities
#'   \item When only one flanking marker is observed, it uses marginal probabilities
#'   \item The calculations incorporate recombination fractions and population-specific parameters
#' }
#' 
#' \strong{Genotype Coding:}
#' The marker types use a coding system where:
#' \itemize{
#'   \item \code{2}: Homozygous for alternative allele
#'   \item \code{1}: Heterozygous
#'   \item \code{0}: Homozygous for reference allele
#'   \item \code{N}: Missing or unknown genotype
#' }
#' 
#' @examples

#' # Calculate probability for RIL population
#' prob_ril <- expected_genotype_dist("00", "RIL", Gn = 2, x = 0.1, y = 0.2)
#' 
#' # Example with different recombination fractions
#' prob_low_rec <- expected_genotype_dist("22", "F2", Gn = 2, x = 0.05, y = 0.05)
#' prob_high_rec <- expected_genotype_dist("22", "F2", Gn = 2, x = 0.3, y = 0.4)
#' 
#' @aliases calculate_expected_genotype_distribution
#' @seealso \code{\link{genotype_freq}} for genotype frequency calculations,
#'          \code{\link{genotype_prob}} for missing genotype imputation
#' 
#' @references
#' 
#' Haldane, J.B.S. (1919). The combination of linkage values and the calculation of 
#' distances between the loci of linked factors. Journal of Genetics, 8(3), 299-309.
#' 
#' @export
expected_genotype_dist <-
function(marType,croType,Gn=2,x,y=0){
  if(Gn < 1){
    stop("Gn should > 0")
  }
  z <- x + y -2*x*y
  if(marType == "22"){
    if (croType=="BCP1"){
      rval <- genotype_freq(croType,Gn+1,8,x,y)/(genotype_freq(croType,Gn+1,8,x,y)+genotype_freq(croType,Gn+1,6,x,y))
    }
    if (croType=="DH"){
      rval <- (1-x-y)/(1-z)
    }
    if (croType=="F2"|croType=="RIL"){
      rval <- ((1-x)*(1-x)*(1-y)*(1-y) - x*x*y*y)/((1-z)*(1-z))
    }
    if (croType=="Fn"){
      rval <- (calculate_genotype_frequencies(croType,Gn,1,x,y) - calculate_genotype_frequencies(croType,Gn,3,x,y))/(calculate_genotype_frequencies(croType,Gn,1,x,y) + calculate_genotype_frequencies(croType,Gn,6,x,y) + calculate_genotype_frequencies(croType,Gn,3,x,y))
    }
  }else if(marType == "21"){
    if (croType=="BCP1"){
      rval <- calculate_genotype_frequencies(croType,Gn+1,7,x,y)/(calculate_genotype_frequencies(croType,Gn+1,7,x,y)+calculate_genotype_frequencies(croType,Gn+1,5,x,y))
    }
    if (croType=="F2"){
      rval <- ((1-x)*(1-x)*(1-y)*y - x*x*y*(1-y))/((1-z)*z)
    }
    if (croType=="Fn"){
      rval <- (calculate_genotype_frequencies(croType,Gn,5,x,y) - calculate_genotype_frequencies(croType,Gn,15,x,y))/(calculate_genotype_frequencies(croType,Gn,5,x,y) + calculate_genotype_frequencies(croType,Gn,7,x,y) + calculate_genotype_frequencies(croType,Gn,11,x,y) + calculate_genotype_frequencies(croType,Gn,15,x,y))
    }
  }else if(marType == "20"){
    if (croType=="DH"){
      rval <- (y-x)/z
    }
    if (croType=="F2"|croType=="RIL"){
      rval <- ((1-x)*(1-x)*y*y - x*x*(1-y)*(1-y))/(z*z)
    }
    if (croType=="Fn"){
      rval <- (calculate_genotype_frequencies(croType,Gn,2,x,y) - calculate_genotype_frequencies(croType,Gn,4,x,y))/(calculate_genotype_frequencies(croType,Gn,2,x,y) + calculate_genotype_frequencies(croType,Gn,12,x,y) + calculate_genotype_frequencies(croType,Gn,4,x,y))
    }
  }else if(marType == "12"){
    if (croType=="BCP1"){
      rval <- calculate_genotype_frequencies(croType,Gn+1,4,x,y)/(calculate_genotype_frequencies(croType,Gn+1,4,x,y)+calculate_genotype_frequencies(croType,Gn+1,2,x,y))
    }
    if (croType=="F2"){
      rval <- ((1-y)*(1-y)*x*(1-x) - x*(1-x)*y*y)/(z*(1-z))
    }
    if (croType=="Fn"){
      rval <- (calculate_genotype_frequencies(croType,Gn,8,x,y) - calculate_genotype_frequencies(croType,Gn,14,x,y))/(calculate_genotype_frequencies(croType,Gn,8,x,y) + calculate_genotype_frequencies(croType,Gn,10,x,y) + calculate_genotype_frequencies(croType,Gn,16,x,y) + calculate_genotype_frequencies(croType,Gn,14,x,y))
    }
  }else if(marType == "11"){
    if (croType=="BCP1"){
      rval <- calculate_genotype_frequencies(croType,Gn+1,3,x,y)/(calculate_genotype_frequencies(croType,Gn+1,3,x,y)+calculate_genotype_frequencies(croType,Gn+1,1,x,y))
    }
    if (croType=="BCP2"){
      rval <- -(calculate_genotype_frequencies(croType,Gn+1,6,x,y)/(calculate_genotype_frequencies(croType,Gn+1,8,x,y)+calculate_genotype_frequencies(croType,Gn+1,6,x,y)))
    }
    if (croType=="F2"|croType=="Fn"){
      rval <- 0
    }
  }else if(marType == "10"){
    if (croType=="BCP2"){
      rval <- -(calculate_genotype_frequencies(croType,Gn+1,5,x,y)/(calculate_genotype_frequencies(croType,Gn+1,7,x,y)+calculate_genotype_frequencies(croType,Gn+1,5,x,y)))
    }
    if (croType=="F2"){
      rval <- (x*(1-x)*y*y -x*(1-x)*(1-y)*(1-y))/(z*(1-z))
    }
    if (croType=="Fn"){
      rval <- (calculate_genotype_frequencies(croType,Gn,14,x,y) - calculate_genotype_frequencies(croType,Gn,8,x,y))/(calculate_genotype_frequencies(croType,Gn,14,x,y) + calculate_genotype_frequencies(croType,Gn,10,x,y) + calculate_genotype_frequencies(croType,Gn,16,x,y) + calculate_genotype_frequencies(croType,Gn,8,x,y))
    }
  }else if(marType == "02"){
    if (croType=="DH"){
      rval <- (x-y)/z
    }
    if (croType=="F2"|croType=="RIL"){
      rval <- ((1-y)*(1-y)*x*x - (1-x)*(1-x)*y*y)/(z*z)
    }
    if (croType=="Fn"){
      rval <- (calculate_genotype_frequencies(croType,Gn,4,x,y) - calculate_genotype_frequencies(croType,Gn,2,x,y))/(calculate_genotype_frequencies(croType,Gn,4,x,y) + calculate_genotype_frequencies(croType,Gn,12,x,y) + calculate_genotype_frequencies(croType,Gn,2,x,y))
    }
  }else if(marType == "01"){
    if (croType=="BCP2"){
      rval <- -(calculate_genotype_frequencies(croType,Gn+1,2,x,y)/(calculate_genotype_frequencies(croType,Gn+1,4,x,y)+calculate_genotype_frequencies(croType,Gn+1,2,x,y)))
    }
    if (croType=="F2"){
      rval <- (x*x*(1-y)*y - (1-x)*(1-x)*(1-y)*y)/((1-z)*z)
    }
    if (croType=="Fn"){
      rval <- (calculate_genotype_frequencies(croType,Gn,15,x,y) - calculate_genotype_frequencies(croType,Gn,5,x,y))/(calculate_genotype_frequencies(croType,Gn,5,x,y) + calculate_genotype_frequencies(croType,Gn,7,x,y) + calculate_genotype_frequencies(croType,Gn,11,x,y) + calculate_genotype_frequencies(croType,Gn,15,x,y))
    }
  }else if(marType == "00"){
    if (croType=="BCP2"){
      rval <- -(calculate_genotype_frequencies(croType,Gn+1,1,x,y)/(calculate_genotype_frequencies(croType,Gn+1,3,x,y)+calculate_genotype_frequencies(croType,Gn+1,1,x,y)))
    }
    if (croType=="DH"){
      rval <- (x+y-1)/(1-z)
    }
    if (croType=="F2"|croType=="RIL"){
      rval <- (x*x*y*y - (1-x)*(1-x)*(1-y)*(1-y))/((1-z)*(1-z))
    }
    if (croType=="Fn"){
      rval <- (calculate_genotype_frequencies(croType,Gn,3,x,y) - calculate_genotype_frequencies(croType,Gn,1,x,y))/(calculate_genotype_frequencies(croType,Gn,1,x,y) + calculate_genotype_frequencies(croType,Gn,6,x,y) + calculate_genotype_frequencies(croType,Gn,3,x,y))
    }
  }else if(marType == "2N"){
    if (croType=="BCP1"){
      rval <- calculate_genotype_frequencies(croType,Gn,4,x)/(calculate_genotype_frequencies(croType,Gn,3,x)+calculate_genotype_frequencies(croType,Gn,4,x))
    }
    if (croType=="DH"){
      rval <- 1-2*x
    }
    if (croType=="Fn"){
      rval <- (calculate_genotype_frequencies(croType,Gn,9,x)-calculate_genotype_frequencies(croType,Gn,7,x))/(calculate_genotype_frequencies(croType,Gn,9,x)+calculate_genotype_frequencies(croType,Gn,8,x)+calculate_genotype_frequencies(croType,Gn,7,x))
    }
    if (croType=="F2"){
      rval <- (1-x)*(1-x) - x*x
    }
    if (croType=="RIL"){
      rval <- ((1-x)*(1-x) - x*x)/((1-x)*(1-x) + x*x)
    }
  }else if(marType == "1N"){
    if (croType=="BCP1"){
      rval <- calculate_genotype_frequencies(croType,Gn,2,x)/(calculate_genotype_frequencies(croType,Gn,1,x)+calculate_genotype_frequencies(croType,Gn,2,x))
    }
    if (croType=="BCP2"){
      rval <- -calculate_genotype_frequencies(croType,Gn,3,x)/(calculate_genotype_frequencies(croType,Gn,4,x)+calculate_genotype_frequencies(croType,Gn,3,x))
    }
    if (croType=="F2"|croType=="Fn"){
      rval <- 0
    }
  }else if(marType == "0N"){
    if (croType=="BCP2"){
      rval <- -calculate_genotype_frequencies(croType,Gn,1,x)/(calculate_genotype_frequencies(croType,Gn,1,x)+calculate_genotype_frequencies(croType,Gn,2,x))
    }
    if (croType=="DH"){
      rval <- 2*x-1
    }
    if(croType=="Fn"){
      rval <- (calculate_genotype_frequencies(croType,Gn,3,x)-calculate_genotype_frequencies(croType,Gn,1,x))/(calculate_genotype_frequencies(croType,Gn,3,x)+calculate_genotype_frequencies(croType,Gn,2,x)+calculate_genotype_frequencies(croType,Gn,1,x))
    }
    if (croType=="F2"){
      rval <- x*x - (1-x)*(1-x)
    }
    if(croType=="RIL"){
      rval <- (x*x - (1-x)*(1-x))/((1-x)*(1-x) + x*x)
    }
  }else if(marType == "N2"){
    if (croType=="BCP1"){
      rval <- calculate_genotype_frequencies(croType,Gn,4,x)/(calculate_genotype_frequencies(croType,Gn,4,x)+calculate_genotype_frequencies(croType,Gn,2,x))
    }
    if (croType=="DH"){
      rval <- 1-2*x
    }
    if (croType=="Fn"){
      rval <- (calculate_genotype_frequencies(croType,Gn,9,x)-calculate_genotype_frequencies(croType,Gn,3,x))/(calculate_genotype_frequencies(croType,Gn,9,x)+calculate_genotype_frequencies(croType,Gn,6,x)+calculate_genotype_frequencies(croType,Gn,3,x))
    }
    if (croType=="F2"){
      rval <- (1-x)*(1-x) - x*x
    }
    if (croType=="RIL"){
      rval <- ((1-x)*(1-x) - x*x)/((1-x)*(1-x) + x*x)
    }
  }else if(marType == "N1"){
    if (croType=="BCP1"){
      rval <- calculate_genotype_frequencies(croType,Gn,3,x)/(calculate_genotype_frequencies(croType,Gn,3,x)+calculate_genotype_frequencies(croType,Gn,1,x))
    }
    if (croType=="BCP2"){
      rval <- -calculate_genotype_frequencies(croType,Gn,2,x)/(calculate_genotype_frequencies(croType,Gn,2,x)+calculate_genotype_frequencies(croType,Gn,4,x))
    }
    if (croType=="F2"|croType=="Fn"){
      rval <- 0
    }
  }else if(marType == "N0"){
    if (croType=="BCP2"){
      rval <- -calculate_genotype_frequencies(croType,Gn,1,x)/(calculate_genotype_frequencies(croType,Gn,1,x)+calculate_genotype_frequencies(croType,Gn,3,x))
    }
    if (croType=="DH"){
      rval <- 2*x-1
    }
    if(croType=="Fn"){
      rval <- (calculate_genotype_frequencies(croType,Gn,7,x)-calculate_genotype_frequencies(croType,Gn,1,x))/(calculate_genotype_frequencies(croType,Gn,1,x)+calculate_genotype_frequencies(croType,Gn,4,x)+calculate_genotype_frequencies(croType,Gn,7,x))
      #error inrval <- (x*x - (1-x)*(1-x))/((1-x)*(1-x) + x*x + (2/(2^(Gn-1)-1))*x*(1-x))
    }
    if (croType=="F2"){
      rval <- x*x - (1-x)*(1-x)
    }
    if(croType=="RIL"){
      rval <- (x*x - (1-x)*(1-x))/((1-x)*(1-x) + x*x)
    }
  }else{
    stop("Wrong mark type for argument marType")
  }
  return(rval)
}

calculate_expected_genotype_distribution <- expected_genotype_dist

#' Genotype Frequency Calculator for Genetic Populations
#' 
#' Calculates genotype frequencies for different cross types and generations in genetic mapping studies.
#' This function implements complex recurrence relations to compute genotype frequencies in various
#' genetic populations including Fn, backcross, and advanced generation populations.
#' 
#' @param cross_type Population type used in the calculation:
#'   \itemize{
#'     \item \code{"Fn"}: F-n generation populations (n > 2)
#'     \item \code{"BCP1"}: Backcross to parent 1 populations
#'     \item \code{"BCP2"}: Backcross to parent 2 populations
#'     \item \code{"F2"}: F2 generation populations
#'     \item \code{"DH"}: Doubled haploid populations
#'     \item \code{"RIL"}: Recombinant inbred line populations
#'   }
#' @param generation Generation number (for example, 2 for F2, 3 for F3). Must be greater than 1.
#' @param genotype_index Index of the genotype class to evaluate.
#'   Valid range depends on `cross_type`:
#'   \itemize{
#'     \item \code{"Fn"}: 1-20 (20 different genotype classes)
#'     \item \code{"BCP1/BCP2"}: 1-8 (8 genotype classes)
#'     \item \code{"F2"}: 1-9 (9 genotype classes)
#'   }
#' @param recomb_aq Recombination fraction between loci A and Q.
#'   Use a value between 0 and 0.5.
#' @param recomb_qb Recombination fraction between loci Q and B.
#'   Use a value between 0 and 0.5. Default is 0.
#' 
#' @return A numeric value representing the frequency of the specified genotype in the given generation.
#'   The result is always between 0 and 1.
#' 
#' @details This function calculates genotype frequencies using sophisticated mathematical models:
#' 
#' \strong{For Fn Populations (y > 0):}
#' \itemize{
#'   \item Handles 20 different genotype classes with complex recurrence relations
#'   \item Accounts for recombination between three loci (A, Q, B)
#'   \item Uses the relationship \eqn{z = x + y - 2xy} for flanking marker recombination
#'   \item Implements generation-by-generation frequency calculations
#' }
#' 
#' \strong{For BCP1 Populations:}
#' \itemize{
#'   \item Handles 8 genotype classes
#'   \item Backcross to the first parent (AAQQBB)
#'   \item Specific recurrence relations for each genotype class
#' }
#' 
#' \strong{For BCP2 Populations:}
#' \itemize{
#'   \item Handles 8 genotype classes
#'   \item Backcross to the second parent (aaqqbb)
#'   \item Different initial conditions and recurrence relations
#' }
#' 
#' \strong{For F2 Populations (y = 0):}
#' \itemize{
#'   \item Handles 9 genotype classes
#'   \item Simplified two-locus model
#'   \item Standard F2 generation frequencies
#' }
#' 
#' \strong{Mathematical Foundation:}
#' The function uses recurrence relations to calculate genotype frequencies across generations.
#' For each generation, the frequency of each genotype class is computed based on the
#' frequencies in the previous generation and the recombination fractions between loci.
#' 
#' @examples
#' # Calculate frequency for F2 generation, genotype 1, with recombination fraction 0.1
#' freq_f2 <- genotype_freq("Fn", generation = 2, genotype_index = 1, recomb_aq = 0.1, recomb_qb = 0)
#' 
#' # Calculate frequency for BCP1 generation, genotype 5, with recombination fractions
#' freq_bcp1 <- genotype_freq(
#'   "BCP1", generation = 3, genotype_index = 5, recomb_aq = 0.15, recomb_qb = 0.25
#' )
#' 
#' # Calculate frequency for Fn generation, genotype 10, with recombination fractions
#' freq_fn <- genotype_freq(
#'   "Fn", generation = 4, genotype_index = 10, recomb_aq = 0.2, recomb_qb = 0.3
#' )
#' 
#' # Calculate frequency for BCP2 generation, genotype 3
#' freq_bcp2 <- genotype_freq(
#'   "BCP2", generation = 2, genotype_index = 3, recomb_aq = 0.1, recomb_qb = 0.2
#' )
#' 
#' # Example with different generation numbers
#' freq_gen3 <- genotype_freq("Fn", generation = 3, genotype_index = 1, recomb_aq = 0.1, recomb_qb = 0)
#' freq_gen5 <- genotype_freq("Fn", generation = 5, genotype_index = 1, recomb_aq = 0.1, recomb_qb = 0)
#' 
#' @aliases calculate_genotype_frequencies
#' 
#' @seealso \code{\link{expected_genotype_dist}} for expected allele genotype distribution calculations
#' 
#' @references
#' 
#' Haldane, J.B.S. (1919). The combination of linkage values and the calculation of 
#' distances between the loci of linked factors. Journal of Genetics, 8(3), 299-309.
#' 
#' @export
genotype_freq <-
function(cross_type, generation = 2, genotype_index, recomb_aq, recomb_qb = 0){
  CrosType <- cross_type
  Gn <- generation
  i <- genotype_index
  x <- recomb_aq
  y <- recomb_qb

  if(!is.numeric(Gn) || length(Gn) != 1 || Gn <= 1){
    stop("generation should be > 1")
  }
  Gn <- as.integer(Gn)
  if(!is.numeric(i) || length(i) != 1 || i < 1){
    stop("genotype_index should be a positive integer")
  }
  i <- as.integer(i)
  if(!is.numeric(x) || length(x) != 1 || x < 0 || x > 0.5){
    stop("recomb_aq should be between 0 and 0.5")
  }
  if(!is.numeric(y) || length(y) != 1 || y < 0 || y > 0.5){
    stop("recomb_qb should be between 0 and 0.5")
  }
  max_index <- if(y != 0) {
    if(CrosType == "Fn") 20 else if(CrosType %in% c("BCP1", "BCP2")) 8 else NA_integer_
  } else {
    if(CrosType == "Fn") 9 else if(CrosType %in% c("BCP1", "BCP2")) 4 else NA_integer_
  }
  if(is.na(max_index)){
    stop("argument cross_type must be 'Fn' or 'BCP1' or 'BCP2'")
  }
  if(i > max_index){
    stop(paste0("genotype_index must be between 1 and ", max_index, " for cross_type=", CrosType))
  }

  if(y != 0){
    z <- x + y -2*x*y
    if(CrosType == "Fn"){
      mtGeFre <- matrix(0, nrow = Gn, ncol = 20)	##mtGeFre: Matrix of Frequency of the Genotypes ,where nrow is Gn and ncol is 1~20.
      mtGeFre[1,20] <- 1	 ##mtGeFre[1,20]=1 means the Frequency of F1 which comes frome AAQQBB * aaqqbb -> AaQqBb equals to 1.
      for (g in 2:Gn){
        #AQB/AQB or aqb/aqb
        mtGeFre[g,1] <- 0.25*(4*mtGeFre[g-1,1]+mtGeFre[g-1,5]+mtGeFre[g-1,6]+(1-y)*(1-y)*mtGeFre[g-1,7]+mtGeFre[g-1,8]+(1-z)*(1-z)*mtGeFre[g-1,9]+
                                (1-x)*(1-x)*mtGeFre[g-1,10]+y*y*mtGeFre[g-1,11]+z*z*mtGeFre[g-1,13]+x*x*mtGeFre[g-1,16]+ x*x*(1-y)*(1-y)*mtGeFre[g-1,17]+
                                x*x*y*y*mtGeFre[g-1,18]+y*y*(1-x)*(1-x)*mtGeFre[g-1,19]+(1-x)*(1-x)*(1-y)*(1-y)*mtGeFre[g-1,20])
        #AQb/AQb or aqB/aqB
        mtGeFre[g,2] <- 0.25*(4*mtGeFre[g-1,2]+mtGeFre[g-1,5]+y*y*mtGeFre[g-1,7]+z*z*mtGeFre[g-1,9]+(1-x)*(1-x)*mtGeFre[g-1,10]+(1-y)*(1-y)*mtGeFre[g-1,11]+
                                mtGeFre[g-1,12]+(1-z)*(1-z)*mtGeFre[g-1,13]+mtGeFre[g-1,14]+x*x*mtGeFre[g-1,16]+ x*x*y*y*mtGeFre[g-1,17]+
                                x*x*(1-y)*(1-y)*mtGeFre[g-1,18]+(1-y)*(1-y)*(1-x)*(1-x)*mtGeFre[g-1,19]+(1-x)*(1-x)*y*y*mtGeFre[g-1,20])
        #AqB/AqB or aQb/aQb
        mtGeFre[g,3] <- 0.25*(4*mtGeFre[g-1,3]+mtGeFre[g-1,6]+y*y*mtGeFre[g-1,7]+(1-z)*(1-z)*mtGeFre[g-1,9]+x*x*mtGeFre[g-1,10]+
                                (1-y)*(1-y)*mtGeFre[g-1,11]+z*z*mtGeFre[g-1,13]+mtGeFre[g-1,14]+mtGeFre[g-1,15]+(1-x)*(1-x)*mtGeFre[g-1,16]+ (1-x)*(1-x)*y*y*mtGeFre[g-1,17]+
                                (1-x)*(1-x)*(1-y)*(1-y)*mtGeFre[g-1,18]+(1-y)*(1-y)*x*x*mtGeFre[g-1,19]+x*x*y*y*mtGeFre[g-1,20])
        #aQB/aQB or Aqb/Aqb
        mtGeFre[g,4] <- 0.25*(4*mtGeFre[g-1,4]+(1-y)*(1-y)*mtGeFre[g-1,7]+mtGeFre[g-1,8]+z*z*mtGeFre[g-1,9]+x*x*mtGeFre[g-1,10]+
                                y*y*mtGeFre[g-1,11]+mtGeFre[g-1,12]+(1-z)*(1-z)*mtGeFre[g-1,13]+mtGeFre[g-1,15]+(1-x)*(1-x)*mtGeFre[g-1,16]+ (1-x)*(1-x)*(1-y)*(1-y)*mtGeFre[g-1,17]+
                                (1-x)*(1-x)*y*y*mtGeFre[g-1,18]+y*y*x*x*mtGeFre[g-1,19]+x*x*(1-y)*(1-y)*mtGeFre[g-1,20])
        #aqB/aqb or AQB/AQb
        mtGeFre[g,5] <- 0.5*(mtGeFre[g-1,5]+y*(1-y)*mtGeFre[g-1,7]+z*(1-z)*mtGeFre[g-1,9]+y*(1-y)*mtGeFre[g-1,11]+z*(1-z)*mtGeFre[g-1,13]+
                               x*x*y*(1-y)*mtGeFre[g-1,17]+x*x*y*(1-y)*mtGeFre[g-1,18]+y*(1-y)*(1-x)*(1-x)*mtGeFre[g-1,19]+(1-x)*(1-x)*y*(1-y)*mtGeFre[g-1,20])
        #aqb/aQb or AqB/AQB
        mtGeFre[g,6] <- 0.5*(mtGeFre[g-1,6]+y*(1-y)*mtGeFre[g-1,7]+x*(1-x)*mtGeFre[g-1,10]+y*(1-y)*mtGeFre[g-1,11]+x*(1-x)*mtGeFre[g-1,16]+
                               x*y*(1-x)*(1-y)*mtGeFre[g-1,17]+x*y*(1-x)*(1-y)*mtGeFre[g-1,18]+x*y*(1-x)*(1-y)*mtGeFre[g-1,19]+x*y*(1-x)*(1-y)*mtGeFre[g-1,20])
        #aQB/aqb or Aqb/AQB
        mtGeFre[g,7] <- 0.5*((1-y)*(1-y)*mtGeFre[g-1,7]+y*y*mtGeFre[g-1,11]+ x*(1-x)*(1-y)*(1-y)*mtGeFre[g-1,17]+x*y*y*(1-x)*mtGeFre[g-1,18]+
                               x*y*y*(1-x)*mtGeFre[g-1,19]+x*(1-x)*(1-y)*(1-y)*mtGeFre[g-1,20])
        #Aqb/aqb or aQB/AQB
        mtGeFre[g,8] <- 0.5*(mtGeFre[g-1,8]+z*(1-z)*mtGeFre[g-1,9]+x*(1-x)*mtGeFre[g-1,10]+z*(1-z)*mtGeFre[g-1,13]+x*(1-x)*mtGeFre[g-1,16]+
                               x*(1-x)*(1-y)*(1-y)*mtGeFre[g-1,17]+x*y*y*(1-x)*mtGeFre[g-1,18]+x*y*y*(1-x)*mtGeFre[g-1,19]+x*(1-x)*(1-y)*(1-y)*mtGeFre[g-1,20])
        #AqB/aqb or aQb/AQB
        mtGeFre[g,9] <- 0.5*((1-z)*(1-z)*mtGeFre[g-1,9]+z*z*mtGeFre[g-1,13]+ x*(1-x)*(1-y)*y*mtGeFre[g-1,17]+x*y*(1-y)*(1-x)*mtGeFre[g-1,18]+
                               x*y*(1-y)*(1-x)*mtGeFre[g-1,19]+x*(1-x)*(1-y)*y*mtGeFre[g-1,20])
        #AQb/aqb or aqB/AQB
        mtGeFre[g,10] <- 0.5*((1-x)*(1-x)*mtGeFre[g-1,10]+x*x*mtGeFre[g-1,16]+ x*x*(1-y)*y*mtGeFre[g-1,17]+x*y*(1-y)*x*mtGeFre[g-1,18]+
                                (1-x)*y*(1-y)*(1-x)*mtGeFre[g-1,19]+(1-x)*(1-x)*(1-y)*y*mtGeFre[g-1,20])
        #aqB/aQb or AqB/AQb
        mtGeFre[g,11] <- 0.5*(y*y*mtGeFre[g-1,7]+(1-y)*(1-y)*mtGeFre[g-1,11]+x*(1-x)*y*y*mtGeFre[g-1,17]+x*(1-y)*(1-y)*(1-x)*mtGeFre[g-1,18]+
                                x*(1-y)*(1-y)*(1-x)*mtGeFre[g-1,19]+x*y*(1-x)*y*mtGeFre[g-1,20])
        #aqB/aQB or Aqb/AQb
        mtGeFre[g,12] <- 0.5*(y*(1-y)*mtGeFre[g-1,7]+x*(1-x)*mtGeFre[g-1,10]+y*(1-y)*mtGeFre[g-1,11]+mtGeFre[g-1,12]+x*(1-x)*mtGeFre[g-1,16]+
                                x*(1-x)*y*(1-y)*mtGeFre[g-1,17]+x*(1-x)*y*(1-y)*mtGeFre[g-1,18]+x*(1-x)*y*(1-y)*mtGeFre[g-1,19]+x*(1-x)*y*(1-y)*mtGeFre[g-1,20])
        #aqB/Aqb or aQB/AQb
        mtGeFre[g,13] <- 0.5*(z*z*mtGeFre[g-1,9]+(1-z)*(1-z)*mtGeFre[g-1,13]+ x*(1-x)*(1-y)*y*mtGeFre[g-1,17]+x*y*(1-y)*(1-x)*mtGeFre[g-1,18]+
                                x*y*(1-y)*(1-x)*mtGeFre[g-1,19]+x*(1-x)*(1-y)*y*mtGeFre[g-1,20])
        #aqB/AqB or aQb/AQb
        mtGeFre[g,14] <- 0.5*(z*(1-z)*mtGeFre[g-1,9]+x*(1-x)*mtGeFre[g-1,10]+z*(1-z)*mtGeFre[g-1,13]+mtGeFre[g-1,14]+x*(1-x)*mtGeFre[g-1,16]+
                                x*(1-x)*y*y*mtGeFre[g-1,17]+x*(1-y)*(1-y)*(1-x)*mtGeFre[g-1,18]+x*(1-y)*(1-y)*(1-x)*mtGeFre[g-1,19]+x*y*y*(1-x)*mtGeFre[g-1,20])
        #aQb/aQB or Aqb/AqB
        mtGeFre[g,15] <- 0.5*(y*(1-y)*mtGeFre[g-1,7]+z*(1-z)*mtGeFre[g-1,9]+y*(1-y)*mtGeFre[g-1,11]+z*(1-z)*mtGeFre[g-1,13]+mtGeFre[g-1,15]+
                                y*(1-x)*(1-x)*(1-y)*mtGeFre[g-1,17]+y*(1-x)*(1-x)*(1-y)*mtGeFre[g-1,18]+x*x*y*(1-y)*mtGeFre[g-1,19]+x*x*y*(1-y)*mtGeFre[g-1,20])
        #aQb/Aqb or aQB/AqB
        mtGeFre[g,16] <- 0.5*(x*x*mtGeFre[g-1,10]+(1-x)*(1-x)*mtGeFre[g-1,16]+ (1-x)*(1-x)*(1-y)*y*mtGeFre[g-1,17]+(1-x)*(1-x)*(1-y)*y*mtGeFre[g-1,18]+
                                x*x*y*(1-y)*mtGeFre[g-1,19]+x*x*(1-y)*y*mtGeFre[g-1,20])
        #aQB/Aqb 
        mtGeFre[g,17] <- 0.5*((1-x)*(1-x)*(1-y)*(1-y)*mtGeFre[g-1,17]+(1-x)*(1-x)*y*y*mtGeFre[g-1,18]+y*y*x*x*mtGeFre[g-1,19]+x*x*(1-y)*(1-y)*mtGeFre[g-1,20])
        #aQb/AqB
        mtGeFre[g,18] <- 0.5*((1-x)*(1-x)*y*y*mtGeFre[g-1,17]+(1-x)*(1-x)*(1-y)*(1-y)*mtGeFre[g-1,18]+(1-y)*(1-y)*x*x*mtGeFre[g-1,19]+x*x*y*y*mtGeFre[g-1,20])
        #aqB/AQb
        mtGeFre[g,19] <- 0.5*(x*x*y*y*mtGeFre[g-1,17]+x*x*(1-y)*(1-y)*mtGeFre[g-1,18]+(1-y)*(1-y)*(1-x)*(1-x)*mtGeFre[g-1,19]+(1-x)*(1-x)*y*y*mtGeFre[g-1,20])
        #aqb/AQB
        mtGeFre[g,20] <- 0.5*(x*x*(1-y)*(1-y)*mtGeFre[g-1,17]+x*x*y*y*mtGeFre[g-1,18]+y*y*(1-x)*(1-x)*mtGeFre[g-1,19]+(1-x)*(1-x)*(1-y)*(1-y)*mtGeFre[g-1,20])
      }
      rval <- mtGeFre[Gn,i]
    }else if(CrosType == "BCP1"){
      mtGeFre <- matrix(0,nrow=Gn,ncol=8)	 ##mtGeFre: Matrix of Frequency of the Genotypes ,where nrow is Gn() and ncol is 1~8().
      mtGeFre[1,1] <- 1											##mtGeFre[1,1]=1 means the Frequency of F1 which comes frome AAQQBB * aaqqbb -> AaQqBb equals to 1.
      for (g in 2:Gn){
        # AQB/aqb
        mtGeFre[g,1] <- 0.5*(1-x)*(1-y)*mtGeFre[g-1,1]
        # AQB/aqB
        mtGeFre[g,2] <- 0.5*((1-x)*y*mtGeFre[g-1,1]+(1-x)*mtGeFre[g-1,2])
        # AQB/aQb
        mtGeFre[g,3] <- 0.5*(x*y*mtGeFre[g-1,1]+(1-z)*mtGeFre[g-1,3])
        # AQB/aQB
        mtGeFre[g,4] <- 0.5*(x*(1-y)*mtGeFre[g-1,1]+x*mtGeFre[g-1,2]+z*mtGeFre[g-1,3]+mtGeFre[g-1,4])
        # AQB/Aqb
        mtGeFre[g,5] <- 0.5*(x*(1-y)*mtGeFre[g-1,1]+(1-y)*mtGeFre[g-1,5])
        # AQB/AqB
        mtGeFre[g,6] <- 0.5*(x*y*mtGeFre[g-1,1]+x*mtGeFre[g-1,2]+y*mtGeFre[g-1,5]+mtGeFre[g-1,6])
        # AQB/AQb
        mtGeFre[g,7] <- 0.5*((1-x)*y*mtGeFre[g-1,1]+z*mtGeFre[g-1,3]+y*mtGeFre[g-1,5]+mtGeFre[g-1,7])
        # AQB/AQB
        mtGeFre[g,8] <- 0.5*((1-x)*(1-y)*mtGeFre[g-1,1]+(1-x)*mtGeFre[g-1,2]+(1-z)*mtGeFre[g-1,3]+mtGeFre[g-1,4]+(1-y)*mtGeFre[g-1,5]+mtGeFre[g-1,6]+mtGeFre[g-1,7])+mtGeFre[g-1,8]
      }
      rval <- mtGeFre[Gn,i]
    }else if(CrosType == "BCP2"){
      mtGeFre <- matrix(0,nrow=Gn,ncol=8)					##mtGeFre: Matrix of Frequency of the Genotypes ,where nrow is Gn() and ncol is 1~8().
      mtGeFre[1,8] <- 1											##mtGeFre[1,1]=1 means the Frequency of F1 which comes frome AAQQBB * aaqqbb -> AaQqBb equals to 1.
      for (g in 2:Gn){
        # aqb/aqb
        mtGeFre[g,1] <- 0.5*((1-x)*(1-y)*mtGeFre[g-1,8]+(1-x)*mtGeFre[g-1,7]+(1-z)*mtGeFre[g-1,6]+mtGeFre[g-1,5]+(1-y)*mtGeFre[g-1,4]+mtGeFre[g-1,3]+mtGeFre[g-1,2])+mtGeFre[g-1,1]
        # aqb/aqB
        mtGeFre[g,2] <- 0.5*((1-x)*y*mtGeFre[g-1,8]+z*mtGeFre[g-1,6]+y*mtGeFre[g-1,4]+mtGeFre[g-1,2])
        # aqb/aQb
        mtGeFre[g,3] <- 0.5*(x*y*mtGeFre[g-1,8]+x*mtGeFre[g-1,7]+y*mtGeFre[g-1,4]+mtGeFre[g-1,3])
        # aqb/aQB
        mtGeFre[g,4] <- 0.5*(x*(1-y)*mtGeFre[g-1,8]+(1-y)*mtGeFre[g-1,4])
        # aqb/Aqb
        mtGeFre[g,5] <- 0.5*(x*(1-y)*mtGeFre[g-1,8]+x*mtGeFre[g-1,7]+z*mtGeFre[g-1,6]+mtGeFre[g-1,5])
        # aqb/AqB
        mtGeFre[g,6] <- 0.5*(x*y*mtGeFre[g-1,8]+(1-z)*mtGeFre[g-1,6])
        # aqb/AQb
        mtGeFre[g,7] <- 0.5*((1-x)*y*mtGeFre[g-1,8]+(1-x)*mtGeFre[g-1,7])
        # aqb/AQB
        mtGeFre[g,8] <- 0.5*(1-x)*(1-y)*mtGeFre[g-1,8]
      }
      rval <- mtGeFre[Gn,i]
    }else{
      stop("argument CrosType must be 'Fn' or 'BCP1' or 'BCP2'")
    }
  }else{
    if(CrosType == "Fn"){
      mtGeFre <- matrix(0,nrow=Gn,ncol=9)	 ##mtGeFre: Matrix of Frequency of the Genotypes ,where nrow is Gn() and ncol is 1~4().
      mtGeFre[1,5] <- 1											##mtGeFre[1,1]=1 means the Frequency of F1 which comes frome AABB * aabb -> AaBb equals to 1.
      for (g in 2:Gn){
        # ab/ab
        mtGeFre[g,1] <- 0.25*(4*mtGeFre[g-1,1]+mtGeFre[g-1,2]+mtGeFre[g-1,4]+(1-x)^2*mtGeFre[g-1,5])
        # aB/ab
        mtGeFre[g,2] <- 0.25*(2*mtGeFre[g-1,2]+x*(1-x)*mtGeFre[g-1,5])
        # aB/aB
        mtGeFre[g,3] <- 0.25*(2*mtGeFre[g-1,2]+4*mtGeFre[g-1,3]+x^2*mtGeFre[g-1,5]+mtGeFre[g-1,6])
        # Ab/ab
        mtGeFre[g,4] <- 0.25*(2*mtGeFre[g-1,4]+x*(1-x)*mtGeFre[g-1,5])
        # AB/ab or Ab/aB
        mtGeFre[g,5] <- 0.25*((1-x)^2+x^2)*mtGeFre[g-1,5]
        # AB/aB
        mtGeFre[g,6] <- 0.25*(2*mtGeFre[g-1,6]+x*(1-x)*mtGeFre[g-1,5])
        # Ab/Ab
        mtGeFre[g,7] <- 0.25*(2*mtGeFre[g-1,8]+4*mtGeFre[g-1,7]+x^2*mtGeFre[g-1,5]+mtGeFre[g-1,4])
        # AB/Ab
        mtGeFre[g,8] <- 0.25*(2*mtGeFre[g-1,8]+x*(1-x)*mtGeFre[g-1,5])
        # ab/ab
        mtGeFre[g,9] <- 0.25*(4*mtGeFre[g-1,9]+mtGeFre[g-1,8]+mtGeFre[g-1,6]+(1-x)^2*mtGeFre[g-1,5])
      }
      rval <- mtGeFre[Gn,i]
    }else if(CrosType == "BCP1"){
      mtGeFre <- matrix(0,nrow=Gn,ncol=4)	 ##mtGeFre: Matrix of Frequency of the Genotypes ,where nrow is Gn() and ncol is 1~4().
      mtGeFre[1,1] <- 1											##mtGeFre[1,1]=1 means the Frequency of F1 which comes frome AABB * aabb -> AaBb equals to 1.
      for (g in 2:Gn){
        # AB/ab
        mtGeFre[g,1] <- 0.5*(1-x)*mtGeFre[g-1,1]
        # AB/aB
        mtGeFre[g,2] <- 0.5*(x*mtGeFre[g-1,1]+mtGeFre[g-1,2])
        # AB/Ab
        mtGeFre[g,3] <- 0.5*(x*mtGeFre[g-1,1]+mtGeFre[g-1,3])
        # AB/AB
        mtGeFre[g,4] <- 0.5*((1-x)*mtGeFre[g-1,1]+mtGeFre[g-1,2]+mtGeFre[g-1,3])+mtGeFre[g-1,4]
      }
      rval <- mtGeFre[Gn,i]
    }else if(CrosType == "BCP2"){
      mtGeFre <- matrix(0,nrow=Gn,ncol=4)	 ##mtGeFre: Matrix of Frequency of the Genotypes ,where nrow is Gn() and ncol is 1~4().
      mtGeFre[1,4] <- 1											##mtGeFre[1,1]=1 means the Frequency of F1 which comes frome AABB * aabb -> AaBb equals to 1.
      for (g in 2:Gn){
        # ab/ab
        mtGeFre[g,1] <- 0.5*(mtGeFre[g-1,2]+mtGeFre[g-1,3]+(1-x)*mtGeFre[g-1,4])+mtGeFre[g-1,1]
        # ab/aB
        mtGeFre[g,2] <- 0.5*(mtGeFre[g-1,2]+x*mtGeFre[g-1,4])
        # ab/Ab
        mtGeFre[g,3] <- 0.5*(mtGeFre[g-1,3]+x*mtGeFre[g-1,4])
        # AB/AB
        mtGeFre[g,4] <- 0.5*(1-x)*mtGeFre[g-1,4]
      }
      rval <- mtGeFre[Gn,i]
    }else{
      stop("argument CrosType must be 'Fn' or 'BCP1' or 'BCP2'")
    }
  }
  return(rval)
}

# backward-compatible alias
calculate_genotype_frequencies <- genotype_freq

#' Haldane Mapping Function
#' 
#' Converts genetic distance in Morgan units to recombination fraction using the Haldane mapping function.
#' This function implements the Haldane mapping function which assumes no interference between crossovers.
#' 
#' @param x A numeric value representing the genetic distance in Morgan units (centimorgans/100).
#'   Must be non-negative.
#' 
#' @return A numeric value representing the recombination fraction (r) between 0 and 0.5.
#' 
#' @details The Haldane mapping function is defined as:
#' \deqn{r = \frac{1}{2}(1 - e^{-2x})}
#' where \eqn{x} is the genetic distance in Morgan units and \eqn{r} is the recombination fraction.
#' 
#' This function assumes no interference between crossovers, meaning that the occurrence
#' of one crossover does not affect the probability of other crossovers occurring nearby.
#' 
#' @examples
#' # Convert 0.1 Morgan (10 cM) to recombination fraction
#' haldane_map(0.1)
#' 
#' # Convert 0.5 Morgan (50 cM) to recombination fraction  
#' haldane_map(0.5)
#' 
#' # Convert 1.0 Morgan (100 cM) to recombination fraction
#' haldane_map(1.0)
#' 
#' @aliases haldane_mapping_function
#' 
#' @references
#' Haldane, J.B.S. (1919). The combination of linkage values and the calculation of 
#' distances between the loci of linked factors. Journal of Genetics, 8(3), 299-309.
#' 
#' @seealso \code{\link{genotype_prob}} for genotype probability calculations
#' 
#' @export
haldane_map <-
function(x){
  r <- 0.5*(1-exp(-2*x))
  return(r)
}

haldane_mapping_function <- haldane_map

#' Calculate Genotype Probabilities and Impute Missing Data
#' 
#' Calculates genotype probabilities and imputes missing genotype data for genetic mapping studies.
#' Supports both Association Mapping (AM) and Linkage Mapping (LM) methods with comprehensive
#' missing data handling and virtual marker creation capabilities.
#' 
#' @param GeneticMap A data frame containing genetic map information with columns:
#'   \itemize{
#'     \item \code{marker}: Marker names (character)
#'     \item \code{chr}: Chromosome numbers (numeric or factor)
#'     \item \code{pos}: Genetic positions in centimorgans (numeric)
#'   }
#' @param GenoData A matrix or data frame containing genotype data with individuals as rows 
#'   and markers as columns. Genotype codes:
#'   \itemize{
#'     \item \code{0}: Homozygous for reference allele
#'     \item \code{1}: Heterozygous
#'     \item \code{2}: Homozygous for alternative allele
#'     \item \code{9} or \code{NA}: Missing data
#'   }
#' @param method Character string specifying the mapping method:
#'   \itemize{
#'     \item \code{"AM"}: Association Mapping - combines map with genotype data
#'     \item \code{"LM"}: Linkage Mapping - performs missing data imputation
#'   }
#' @param croType Character string specifying the cross type for LM method. Required when \code{method="LM"}:
#'   \itemize{
#'     \item \code{"RIL"}: Recombinant Inbred Lines
#'     \item \code{"DH"}: Doubled Haploids
#'     \item \code{"F2"}: F2 population
#'     \item \code{"BCP1"}: Backcross to Parent 1
#'     \item \code{"BCP2"}: Backcross to Parent 2
#'     \item \code{"Fn"}: Advanced generation (n > 2)
#'   }
#' @param steps Numeric value specifying the step size (in cM) for virtual marker creation in LM.
#'   If 0, no virtual markers are created. Default is 0.
#' @param Gn Numeric value specifying the generation number for advanced populations.
#'   Must be greater than 1. Default is 2.
#' 
#' @return A data frame containing the processed genotype data with genetic map information
#'   and calculated genotype probabilities. The structure depends on the method:
#'   \itemize{
#'     \item \strong{AM method}: Original data combined with genetic map
#'     \item \strong{LM method}: Imputed genotype data with probabilities and optional virtual markers
#'   }
#' 
#' @details This function provides comprehensive genotype data processing for genetic mapping:
#' 
#' \strong{Association Mapping (AM):}
#' \itemize{
#'   \item Simply combines genetic map with genotype data
#'   \item Converts missing values to code 9
#'   \item No imputation performed
#' }
#' 
#' \strong{Linkage Mapping (LM):}
#' \itemize{
#'   \item Validates genotype codes based on cross type expectations
#'   \item Imputes missing genotypes using flanking marker information
#'   \item Creates virtual markers at specified intervals when \code{steps > 0}
#'   \item Uses Haldane mapping function for recombination calculations
#'   \item Applies cross-type specific genotype probability calculations
#' }
#' 
#' \strong{Cross Type Genotype Expectations:}
#' \itemize{
#'   \item \code{RIL/DH}: Only codes 0 and 2 allowed
#'   \item \code{F2}: Codes 0, 1, and 2 allowed
#'   \item \code{BCP1/BCP2}: Codes 0, 1, and 2 with specific constraints
#'   \item \code{Fn}: Codes 0, 1, and 2 for advanced generations
#' }
#' 
#' \strong{Virtual Marker Creation:}
#' When \code{steps > 0}, virtual markers are created at regular intervals between
#' existing markers to improve mapping resolution and handle large gaps in the genetic map.
#' 
#' @examples
#' # Example genetic map
#' genetic_map <- data.frame(
#'   marker = c("M1", "M2", "M3", "M4", "M5"),
#'   chr = c(1, 1, 1, 1, 1),
#'   pos = c(0, 10, 25, 40, 50)
#' )
#' 
#' # Example genotype data with missing values
#' geno_data <- matrix(
#'   c(2, 0, NA, 1, 2,
#'     1, 0, 2, NA, 0,
#'     2, 1, 0, 2, 1,
#'     0, NA, 1, 0, 2),
#'   nrow = 4, ncol = 5,
#'   dimnames = list(c("Ind1", "Ind2", "Ind3", "Ind4"), 
#'                   c("M1", "M2", "M3", "M4", "M5"))
#' )
#' 
#' # Association mapping (no imputation)
#' result_am <- genotype_prob(genetic_map, geno_data, method = "AM")
#' result_am
#' 
#' # Linkage mapping with virtual marker creation
#' result_lm_vm <- genotype_prob(genetic_map, geno_data, method = "LM", 
#'                            croType = "F2", steps = 5)
#' result_lm_vm
#' 
#' @seealso \code{\link{haldane_map}} for recombination fraction calculations,
#'          \code{\link{expected_genotype_dist}} for expected genotype probabilities
#' 
#' @references
#' Haldane, J.B.S. (1919). The combination of linkage values and the calculation of 
#' distances between the loci of linked factors. Journal of Genetics, 8(3), 299-309.
#' 
#' @export
genotype_prob <- function(GeneticMap,GenoData,method,croType=NULL,steps=0,Gn=2){
    if(nrow(GeneticMap) != ncol(GenoData)){
      stop("Marker No. in geneticMap and GenoData must be equal\n")
    }
    if(method=="AM"){
      GenoData[is.na(GenoData)] <- 9
      tGeneticMapt <- t(GeneticMap[,-1])
      colnames(tGeneticMapt) <- colnames(GenoData)
      RetValue <- rbind(tGeneticMapt,GenoData)
    }else if(method=="LM"){
      if(steps < 0){
        stop("Parameter steps must be postive\n")
      }
      if(croType=="RIL" | croType=="DH") {
        if(!all(is.na(GenoData[!GenoData==2 & !GenoData==0] ))){
          warning("GenoData numeric should be 0 or 2 for RIL and DH population, other numeric has been transformed to NA\n")
          GenoData[!GenoData==2 & !GenoData==0] <- NA
        }
      }
      if(croType=="F2") {
        if(!all(is.na(GenoData[!GenoData==2 & !GenoData==1 & !GenoData==0] ))){
          warning("GenoData numeric should be 0 1 2 for F2 population, other numeric has been transformed to NA\n")
          GenoData[!GenoData==2 & !GenoData==1 & !GenoData==0] <- NA
        }
      }
      if(croType=="BCP1" | croType=="BCP2") {
        if(!all(is.na(GenoData[!GenoData==2 & !GenoData==1 & !GenoData==0] ))){
          warning("GenoData numeric should be 1 2 and 1 0 for BCP1 and BCP2 population respectively, other numeric has been transformed to NA\n")
          GenoData[!GenoData==2 & !GenoData==1 & !GenoData==0] <- NA
        }
      }
      NoChr <- nlevels(as.factor(GeneticMap$chr))
      ConChr <- levels(as.factor(GeneticMap$chr))
      rownames(GeneticMap) <- GeneticMap[,1]
      #RMapList
      RMapList <- NULL
      for (i in 1:NoChr){
        tempV <- subset(GeneticMap,GeneticMap$chr==ConChr[i])
        RMapList[[i]] <- tempV$pos
        names(RMapList[[i]]) <- tempV$marker
      }
      if(steps==0){
        VMmatrix <- GenoData
        nObs <- nrow(GenoData)
        for(i in 1:NoChr){
          Chr_R <- RMapList[[i]]
          for(nS in 1:nObs){
            for(j in 1:length(Chr_R)){
              if(GenoData[nS,names(Chr_R)[j]] %in% NA){
                jL <- j
                jR <- j
                while(jR < length(Chr_R) & GenoData[nS,names(Chr_R)[jR]] %in% NA){
                  jR <- jR+1
                }
                while(jL > 1 & GenoData[nS,names(Chr_R)[jL]] %in% NA){
                  jL <- jL-1
                }
                if(jL==1 & GenoData[nS,names(Chr_R)[jL]] %in% NA){
                  xb <- as.numeric(Chr_R[jR]-Chr_R[j])
                  rb <- haldane_mapping_function(xb/100)
                  if(croType != "BCP2" & GenoData[nS,names(Chr_R)[jR]] %in% 2){
                    VMmatrix[nS,names(Chr_R)[j]] <- round(calculate_expected_genotype_distribution("N2",croType,Gn,rb),digits=3)+1
                  }
                  if(croType != "DH" & croType != "RIL"& GenoData[nS,names(Chr_R)[jR]] %in% 1){
                    VMmatrix[nS,names(Chr_R)[j]] <- round(calculate_expected_genotype_distribution("N1",croType,Gn,rb),digits=3)+1
                  }
                  if(croType != "BCP1"& GenoData[nS,names(Chr_R)[jR]] %in% 0){
                    VMmatrix[nS,names(Chr_R)[j]] <- round(calculate_expected_genotype_distribution("N0",croType,Gn,rb),digits=3)+1
                  }
                }
                if(jR==length(Chr_R) & GenoData[nS,names(Chr_R)[jR]] %in% NA){
                  xa <- as.numeric(Chr_R[j]-Chr_R[jL])
                  ra <- haldane_mapping_function(xa/100)
                  if (croType != "BCP2" & GenoData[nS,names(Chr_R)[jL]] %in% 2){
                    VMmatrix[nS,names(Chr_R)[j]] <- round(calculate_expected_genotype_distribution("2N",croType,Gn,ra),digits=3)+1
                  }
                  if (croType != "DH" & croType != "RIL" & GenoData[nS,names(Chr_R)[jL]] %in% 1){
                    VMmatrix[nS,names(Chr_R)[j]] <- round(calculate_expected_genotype_distribution("1N",croType,Gn,ra),digits=3)+1
                  }
                  if (croType != "BCP1" & GenoData[nS,names(Chr_R)[jL]] %in% 0){
                    VMmatrix[nS,names(Chr_R)[j]] <- round(calculate_expected_genotype_distribution("0N",croType,Gn,ra),digits=3)+1
                  }
                }
                #No 1st and last missing marker
                if(!GenoData[nS,names(Chr_R)[jL]] %in% NA & !GenoData[nS,names(Chr_R)[jR]] %in% NA){
                  #else{
                  xa <- as.numeric(Chr_R[j]-Chr_R[jL])
                  xb <- as.numeric(Chr_R[jR]-Chr_R[j])
                  ra <- haldane_mapping_function(xa/100)
                  rb <- haldane_mapping_function(xb/100)
                  if (croType != "BCP2" & GenoData[nS,names(Chr_R)[jL]] %in% 2 & GenoData[nS,names(Chr_R)[jR]] %in% 2){
                    VMmatrix[nS,names(Chr_R)[j]] <- round(calculate_expected_genotype_distribution("22",croType,Gn,ra,rb),digits=3)+1
                  }
                  if (croType != "BCP2" & croType != "DH" & croType != "RIL" & GenoData[nS,names(Chr_R)[jL]] %in% 2 & GenoData[nS,names(Chr_R)[jR]] %in% 1){
                    VMmatrix[nS,names(Chr_R)[j]] <- round(calculate_expected_genotype_distribution("21",croType,Gn,ra,rb),digits=3)+1
                  }
                  if (croType != "BCP2" & croType != "BCP1" & GenoData[nS,names(Chr_R)[jL]] %in% 2 & GenoData[nS,names(Chr_R)[jR]] %in% 0){
                    VMmatrix[nS,names(Chr_R)[j]] <- round(calculate_expected_genotype_distribution("20",croType,Gn,ra,rb),digits=3)+1
                  }
                  if (croType != "BCP2" & croType != "DH" & croType != "RIL" & GenoData[nS,names(Chr_R)[jL]] %in% 1 & GenoData[nS,names(Chr_R)[jR]] %in% 2){
                    VMmatrix[nS,names(Chr_R)[j]] <- round(calculate_expected_genotype_distribution("12",croType,Gn,ra,rb),digits=3)+1
                  }
                  if (croType != "DH" & croType != "RIL" & GenoData[nS,names(Chr_R)[jL]] %in% 1 & GenoData[nS,names(Chr_R)[jR]] %in% 1){
                    VMmatrix[nS,names(Chr_R)[j]] <- round(calculate_expected_genotype_distribution("11",croType,Gn,ra,rb),digits=3)+1
                  }
                  if (croType != "BCP1" & croType != "DH" & croType != "RIL" & GenoData[nS,names(Chr_R)[jL]] %in% 1 & GenoData[nS,names(Chr_R)[jR]] %in% 0){
                    VMmatrix[nS,names(Chr_R)[j]] <- round(calculate_expected_genotype_distribution("10",croType,Gn,ra,rb),digits=3)+1
                  }
                  if (croType != "BCP2" & croType != "BCP1" & GenoData[nS,names(Chr_R)[jL]] %in% 0 & GenoData[nS,names(Chr_R)[jR]] %in% 2){
                    VMmatrix[nS,names(Chr_R)[j]] <- round(calculate_expected_genotype_distribution("02",croType,Gn,ra,rb),digits=3)+1
                  }
                  if (croType != "BCP1" & croType != "DH" & croType != "RIL" & GenoData[nS,names(Chr_R)[jL]] %in% 0 & GenoData[nS,names(Chr_R)[jR]] %in% 1){
                    VMmatrix[nS,names(Chr_R)[j]] <- round(calculate_expected_genotype_distribution("01",croType,Gn,ra,rb),digits=3)+1
                  }
                  if (croType != "BCP1" & GenoData[nS,names(Chr_R)[jL]] %in% 0 & GenoData[nS,names(Chr_R)[jR]] %in% 0){
                    VMmatrix[nS,names(Chr_R)[j]] <- round(calculate_expected_genotype_distribution("00",croType,Gn,ra,rb),digits=3)+1
                  }
                } 
              }
            }#j
          }#nObs
        }#NoChr
        VMmatrix <- round(VMmatrix,digits = 3)
        RetValue <- rbind(t(GeneticMap[,-1]),VMmatrix)
      }else{
        #VMapList
        VMapList <- NULL
        geneticMap <- NULL
        for(i in 1:NoChr){
          #i=1
          chi <- RMapList[[i]]
          newCh <- chi[1]
          n <- 0
          for(j in 2:length(chi)){
            #j=8
            gap <- chi[j]-chi[j-1]
            cushion <- chi[j-1]
            if(gap > steps){
              nVM <- trunc(gap/steps)
              perdist <- gap/(nVM+1)
              vecVM <- round(cushion + perdist*c(1:nVM),digits=3)
              names(vecVM) <- paste("V",i,"_",c((n+j):(n+j+nVM-1)),sep="") 
              newCh <- append(newCh,vecVM)
              n <- n+nVM
            }#gap
            newCh <- append(newCh,chi[j])
          }#j
          subMap <- matrix(NA,2,length(newCh))
          subMap <- as.data.frame(subMap)
          colnames(subMap) <- names(newCh)
          #subMap <- as.data.frame(subMap)
          rownames(subMap) <- c("chr","pos")
          subMap[1,] <- ConChr[i]
          subMap[2,] <- newCh
          if(i==1){
            geneticMap <- subMap
          }else{
            geneticMap <- cbind(geneticMap,subMap)
          }
          VMapList[i] <- list(newCh)
        }#i
        #VMapLine
        NoMM <- 0      #No. of Marker on the Map
        LineDis <- 0   #Line distance
        VMapLine <- NULL     #mapline including Virtual markers
        FMCh <- NULL   #vector of First marker of each chromosome 
        EMCh <- NULL   #vector of End marker of each chromosome 
        for (i in 1:NoChr) {
          chi <- VMapList[[i]]
          StartP <- chi[1]
          for (j in 1:length(chi)) {
            NoMM <- NoMM + 1
            VMapLine <- append(VMapLine, (chi[j]+LineDis))
            names(VMapLine)[NoMM] <- names(chi)[j]
            if (j == 1) {
              FMCh <- append(FMCh,names(chi)[j])
            }else if (j == length(chi)) {
              EMCh <- append(EMCh,names(chi)[j])
              LineDis <- VMapLine[NoMM]
            }
          }
          #LineDis <- LineDis + 20
        }
        nM_V <- NoMM          #nM_V: No. of Markers
        nObs <- nrow(GenoData)
        VMmatrix <- matrix(NA,nObs,nM_V)
        colnames(VMmatrix) <- names(VMapLine)
        rownames(VMmatrix) <- rownames(GenoData)
        for(j in 1:ncol(GenoData)){
          VMmatrix[,colnames(GenoData)[j]] <- GenoData[,j]
        }
        for(i in 1:NoChr){
          #i=1
          Chr_R <- RMapList[[i]]
          Chr_V <- VMapList[[i]]
          for(jr in 2:length(Chr_R)){
            #jr=8
            nL <- which(names(Chr_V) %in% names(Chr_R)[jr-1])
            nR <- which(names(Chr_V) %in% names(Chr_R)[jr])
            for(nS in 1:nObs){
              #nS=2
              jR <- jr  #j=1
              jL <- jr-1
              while(jR < length(Chr_R) & GenoData[nS,names(Chr_R)[jR]] %in% NA){
                jR <- jR+1
              }
              while(jL > 1 & GenoData[nS,names(Chr_R)[jL]] %in% NA){
                jL <- jL-1
              }
              for(jv in nL:nR){
                #jv = 66
                if(VMmatrix[nS,names(Chr_V)[jv]] %in% NA){
                  if(jL==1 & jR ==length(Chr_R)){
                    stop("marker data without valid marker")
                  }
                  #the 1st missing marker 
                  if(jL==1 & GenoData[nS,names(Chr_R)[jL]] %in% NA){
                    xb <- as.numeric(Chr_R[jR]-Chr_V[jv])
                    rb <- haldane_mapping_function(xb/100)
                    if(croType != "BCP2" & GenoData[nS,names(Chr_R)[jR]] %in% 2){
                      VMmatrix[nS,names(Chr_V)[jv]] <- round(calculate_expected_genotype_distribution("N2",croType,Gn,rb),digits=3)+1
                    }
                    if(croType != "DH" & croType != "RIL"& GenoData[nS,names(Chr_R)[jR]] %in% 1){
                      VMmatrix[nS,names(Chr_V)[jv]] <- round(calculate_expected_genotype_distribution("N1",croType,Gn,rb),digits=3)+1
                    }
                    if(croType != "BCP1"& GenoData[nS,names(Chr_R)[jR]] %in% 0){
                      VMmatrix[nS,names(Chr_V)[jv]] <- round(calculate_expected_genotype_distribution("N0",croType,Gn,rb),digits=3)+1
                    }
                  }
                  #the last missing marker
                  if(jR==length(Chr_R) & GenoData[nS,names(Chr_R)[jR]] %in% NA){
                    xa <- as.numeric(Chr_V[jv]-Chr_R[jL])
                    ra <- haldane_mapping_function(xa/100)
                    if (croType != "BCP2" & GenoData[nS,names(Chr_R)[jL]] %in% 2){
                      VMmatrix[nS,names(Chr_V)[jv]] <- round(calculate_expected_genotype_distribution("2N",croType,Gn,ra),digits=3)+1
                    }
                    if (croType != "DH" & croType != "RIL" & GenoData[nS,names(Chr_R)[jL]] %in% 1){
                      VMmatrix[nS,names(Chr_V)[jv]] <- round(calculate_expected_genotype_distribution("1N",croType,Gn,ra),digits=3)+1
                    }
                    if (croType != "BCP1" & GenoData[nS,names(Chr_R)[jL]] %in% 0){
                      VMmatrix[nS,names(Chr_V)[jv]] <- round(calculate_expected_genotype_distribution("0N",croType,Gn,ra),digits=3)+1
                    }
                  }
                  #No 1st and last missing marker
                  if(!GenoData[nS,names(Chr_R)[jL]] %in% NA & !GenoData[nS,names(Chr_R)[jR]] %in% NA){
                    #else{
                    xa <- as.numeric(Chr_V[jv]-Chr_R[jL])
                    xb <- as.numeric(Chr_R[jR]-Chr_V[jv])
                    ra <- haldane_mapping_function(xa/100)
                    rb <- haldane_mapping_function(xb/100)
                    if (croType != "BCP2" & GenoData[nS,names(Chr_R)[jL]] %in% 2 & GenoData[nS,names(Chr_R)[jR]] %in% 2){
                      VMmatrix[nS,names(Chr_V)[jv]] <- round(calculate_expected_genotype_distribution("22",croType,Gn,ra,rb),digits=3)+1
                    }
                    if (croType != "BCP2" & croType != "DH" & croType != "RIL" & GenoData[nS,names(Chr_R)[jL]] %in% 2 & GenoData[nS,names(Chr_R)[jR]] %in% 1){
                      VMmatrix[nS,names(Chr_V)[jv]] <- round(calculate_expected_genotype_distribution("21",croType,Gn,ra,rb),digits=3)+1
                    }
                    if (croType != "BCP2" & croType != "BCP1" & GenoData[nS,names(Chr_R)[jL]] %in% 2 & GenoData[nS,names(Chr_R)[jR]] %in% 0){
                      VMmatrix[nS,names(Chr_V)[jv]] <- round(calculate_expected_genotype_distribution("20",croType,Gn,ra,rb),digits=3)+1
                    }
                    if (croType != "BCP2" & croType != "DH" & croType != "RIL" & GenoData[nS,names(Chr_R)[jL]] %in% 1 & GenoData[nS,names(Chr_R)[jR]] %in% 2){
                      VMmatrix[nS,names(Chr_V)[jv]] <- round(calculate_expected_genotype_distribution("12",croType,Gn,ra,rb),digits=3)+1
                    }
                    if (croType != "DH" & croType != "RIL" & GenoData[nS,names(Chr_R)[jL]] %in% 1 & GenoData[nS,names(Chr_R)[jR]] %in% 1){
                      VMmatrix[nS,names(Chr_V)[jv]] <- round(calculate_expected_genotype_distribution("11",croType,Gn,ra,rb),digits=3)+1
                    }
                    if (croType != "BCP1" & croType != "DH" & croType != "RIL" & GenoData[nS,names(Chr_R)[jL]] %in% 1 & GenoData[nS,names(Chr_R)[jR]] %in% 0){
                      VMmatrix[nS,names(Chr_V)[jv]] <- round(calculate_expected_genotype_distribution("10",croType,Gn,ra,rb),digits=3)+1
                    }
                    if (croType != "BCP2" & croType != "BCP1" & GenoData[nS,names(Chr_R)[jL]] %in% 0 & GenoData[nS,names(Chr_R)[jR]] %in% 2){
                      VMmatrix[nS,names(Chr_V)[jv]] <- round(calculate_expected_genotype_distribution("02",croType,Gn,ra,rb),digits=3)+1
                    }
                    if (croType != "BCP1" & croType != "DH" & croType != "RIL" & GenoData[nS,names(Chr_R)[jL]] %in% 0 & GenoData[nS,names(Chr_R)[jR]] %in% 1){
                      VMmatrix[nS,names(Chr_V)[jv]] <- round(calculate_expected_genotype_distribution("01",croType,Gn,ra,rb),digits=3)+1
                    }
                    if (croType != "BCP1" & GenoData[nS,names(Chr_R)[jL]] %in% 0 & GenoData[nS,names(Chr_R)[jR]] %in% 0){
                      VMmatrix[nS,names(Chr_V)[jv]] <- round(calculate_expected_genotype_distribution("00",croType,Gn,ra,rb),digits=3)+1
                    }
                }
              }
            }#jv
          }#nObs
        }#jr
      }#NoChr
      VMmatrix <- round(VMmatrix,digits = 3)
      RetValue <- rbind(geneticMap,VMmatrix)
    }#steps
  }else{
    stop("method must be AM or LM")
  }
  return(as.data.frame(RetValue))
}

#' Statistical Analysis of Phenotype Data
#' 
#' Performs comprehensive statistical analysis on phenotype data including normality tests, 
#' analysis of variance (ANOVA), and least squares means calculations for genetic studies.
#' 
#' @param phenoData A data frame containing phenotype data with at least 5 columns.
#'   The first 4 columns must be: Environment (E), Block (B), Repetition (R), and Genotype (G).
#'   Additional columns contain trait measurements for statistical analysis.
#' @param defineForm Optional character vector of custom formula strings for statistical analysis.
#'   If NULL, default formulas are automatically generated based on the data structure.
#' @param effNotation Character string specifying the effect notation for least squares means.
#'   Default is "G" for genotype effect.
#' 
#' @return A list containing comprehensive statistical analysis results for each trait:
#'   \item{normality_test}{Results of Shapiro-Wilk normality test}
#'   \item{formula}{Formula used for the statistical model}
#'   \item{ANOVA}{Complete analysis of variance results}
#'   \item{lsmeans}{Least squares means for genotypes with standard errors}
#' 
#' @details This function performs a comprehensive statistical analysis pipeline:
#' 
#' \enumerate{
#'   \item \strong{Normality Test}: Uses Shapiro-Wilk test to assess normality of each trait
#'   \item \strong{Model Selection}: Automatically generates appropriate statistical models based on data structure
#'   \item \strong{ANOVA}: Performs analysis of variance to test significance of effects
#'   \item \strong{Least Squares Means}: Calculates adjusted means for genotypes
#' }
#' 
#' \strong{Automatic Model Generation:}
#' The function automatically generates appropriate formulas based on the experimental design:
#' \itemize{
#'   \item Multiple environments and blocks: \code{trait ~ E + G + E:G + B\%in\%E}
#'   \item Multiple environments only: \code{trait ~ E*G}
#'   \item Custom formulas: User-defined formulas when \code{defineForm} is provided
#' }
#' 
#' \strong{Data Requirements:}
#' The phenotype data must have the following structure:
#' \itemize{
#'   \item Column 1: Environment (E) - factor variable
#'   \item Column 2: Block (B) - factor variable  
#'   \item Column 3: Repetition (R) - factor variable
#'   \item Column 4: Genotype (G) - factor variable
#'   \item Columns 5+: Trait measurements - numeric variables
#' }
#' 
#' @examples
#' 
#' # Example with multiple environments and blocks
#' pheno_data <- data.frame(
#'   E = rep(c("Env1", "Env2", "Env3"), each = 60),
#'   B = rep(c("Block1", "Block2"), each = 30, times = 3),
#'   R = rep(1:5, 36),
#'   G = rep(1:12, 15),
#'   Height = rnorm(180, 175, 8),
#'   Weight = rnorm(180, 75, 12)
#' )
#' 
#' # Perform statistical analysis with default formulas
#' results <- pheno_stats(pheno_data)
#' 
#' # View normality test results
#' results$Height$normality_test
#' 
#' # View ANOVA results
#' results$Height$ANOVA
#' 
#' # View least squares means
#' results$Height$lsmeans
#' 
#' # Example with custom formulas
#' custom_formulas <- c(
#'   "Height ~ E + G + E:G + B%in%E",
#'   "Weight ~ E + G + E:G"
#' )
#' results_custom <- pheno_stats(pheno_data, defineForm = custom_formulas)
#'
#' @importFrom stats shapiro.test lm anova predict as.formula
#' 
#' @references
#' Shapiro, S.S. and Wilk, M.B. (1965). An analysis of variance test for normality. 
#' Biometrika, 52(3-4), 591-611.
#' 
#' @seealso \code{\link{joint_map}} for joint mapping analysis
#' 
#' @export
pheno_stats <-
function(phenoData,defineForm=NULL,effNotation="G"){
  #Environment(E) effect: multiple year and location;  Block(B) effect: block effect nested in every Environment;  Repetition(R):
  if(ncol(phenoData)<5){
    stop("The number of column should be more than 4!")
  }else{
    colnames(phenoData)[1:4] <- c("E","B","R","G")
    phenoData[,"E"] <- as.factor(phenoData[,"E"])
    phenoData[,"B"] <- as.factor(phenoData[,"B"])
    phenoData[,"G"] <- as.factor(phenoData[,"G"])
  }
  Phelist <- list(list())
  #1 normality abalysis for phenotype
  for(n in 1:(ncol(phenoData)-4)){
    normtest <- shapiro.test(phenoData[,4+n])
    if(normtest$"p.value" < 0.05){
      warning(paste("P value of shapiro.test for ",colnames(phenoData)[4+n]," is ",normtest$"p.value",sep=""))
    }
    Phelist[[colnames(phenoData)[4+n]]]["normality_test"] <- list(normtest)
  }
  #2-3 anova and lsmeans
  if(is.null(defineForm)){
    Form <- NULL
    if(nlevels(phenoData[,"E"])>1){
      if(nlevels(phenoData[,"B"])>1){
        for(i in 1:(ncol(phenoData)-4)){
          Form[i] <- as.character(paste(colnames(phenoData)[4+i],"~ E + G + E:G + B%in%E",sep=""))
        }
      }else{
        Form[i] <- as.character(paste(colnames(phenoData)[4+i],"~ E*G",sep=""))
      }
      #}else if(nlevels(phenoData[,"E"]) == 1 && nlevels(phenoData[,"B"])>1){
      #Form[i] <- as.character(paste(colnames(phenoData)[4+i],"~ G + B",sep=""))
    }else{
      stop("Phenotype data is not multiple year-location data!")
    }
    for(j in 1:(ncol(phenoData)-4)){
      lm0 <- lm(as.formula(Form[j]),data=phenoData)
      ano0 <- anova(lm0)
      lms <- as.data.frame(predict(lsmeans::lsmeans(lm0, "G")))
      colnames(lms) <- colnames(phenoData)[4+j]
      Phelist[[colnames(lms)]]["formula"] <- list(defineForm[j])
      Phelist[[colnames(lms)]]["ANOVA"] <- list(ano0)
      Phelist[[colnames(lms)]]["lsmeans"] <- list(lms)
    }
  }else{
    for(j in 1:length(defineForm)){
      lm0 <- lm(as.formula(defineForm[j]),data=phenoData)
      ano0 <- anova(lm0)
      lms <- as.data.frame(predict(lsmeans::lsmeans(lm0,effNotation)))
      #colnames(lms) <- gsub('\\)|\\(', '', defineForm[j][2])
      colnames(lms) <- unlist(strsplit(defineForm[j], "~"))[1]
      Phelist[[colnames(lms)]]["formula"] <- list(defineForm[j])
      Phelist[[colnames(lms)]]["ANOVA"] <- list(ano0)
      Phelist[[colnames(lms)]]["lsmeans"] <- list(lms)
    }
  }
  #4 varcomponent
  return(Phelist)
}

#' Fit the stepwise “skeleton” model for joint mapping
#'
#' Runs \code{StepReg::stepwise()} with entry/stay levels set from a permutation
#' p-value threshold, and returns the selected best linear model. The result is
#' typically passed to \code{\link{joint_map}()} as the \code{skeleton} argument.
#'
#' @param formula A model formula (same as used in \code{\link{permutation_test}}).
#' @param data A data frame containing all variables in \code{formula}.
#' @param type Model type passed to \code{StepReg::stepwise()}. Default is
#'   \code{"linear"}.
#' @param strategy Stepwise strategy, for example \code{"forward"},
#'   \code{"backward"}, or \code{"bidirection"}. Default is \code{"bidirection"}.
#' @param metric Information criterion or selection metric in StepReg, for
#'   example \code{"SL"}. Default is \code{"SL"}.
#' @param include Optional character vector of terms to keep in the stepwise
#'   search, passed to \code{StepReg::stepwise()}. Default is \code{NULL}.
#' @param cut_off_list A list with a component \code{cut_off} that includes a
#'   named entry \code{"pvalue"} (for example, the return value of
#'   \code{\link{permutation_test}}). This value is used for both \code{sle} and
#'   \code{sls} in the stepwise call.
#'
#' @return The best model object for the chosen \code{strategy} and
#'   \code{metric} (an element of the \code{StepReg::stepwise()} result,
#'   typically with a \code{call} and \code{call$formula}).
#'
#' @seealso \code{\link{joint_map}}, \code{\link{permutation_test}}
#'
#' @importFrom StepReg stepwise
#'
#' @export
skeletion_build <- function(formula, data, type = "linear", strategy = "bidirection",
                            metric = "SL", include = NULL, cut_off_list) {
  cut_off_pvalue <- cut_off_list$cut_off["pvalue"]
  stepwise_model <- stepwise(
    formula, data, type = type, strategy = strategy, metric = metric,
    include = include, sle = cut_off_pvalue, sls = cut_off_pvalue
  )
  best_model <- stepwise_model[[strategy]][[metric]]
  best_model
}

#' @rdname skeletion_build
#' @aliases skeleton_build
#' @export
skeleton_build <- skeletion_build

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/joint_map.R
\name{joint_map}
\alias{joint_map}
\title{Joint Mapping Analysis for Multiple Traits}
\usage{
joint_map(formula, data, skeleton, include, cut_off_list)
}
\arguments{
\item{formula}{A model formula defining the full set of candidate linear-model terms
(for example, factor population effects and marker-by-population interactions).
Term labels from this formula are compared to the selected model in \code{skeleton}.}

\item{data}{A data frame containing all variables used in \code{formula} and in
\code{skeleton$call$formula}.}

\item{skeleton}{A fitted model object (typically the final model from stepwise
selection) that contains \code{skeleton$call$formula}. Commonly produced by
\code{skeletion_build()} using the same \code{formula}, \code{data},
\code{include}, and \code{cut_off_list}.}

\item{include}{Optional character vector of predictor names that should be treated
as grouping variables (coerced to factor when \code{formula} contains \code{:}
interactions). If \code{NULL}, interaction-based inclusion may be inferred from
term labels. Matches the \code{include} argument used when fitting \code{skeleton}.}

\item{cut_off_list}{A list whose component \code{cut_off} includes a named element
\code{"pvalue"} (for example, output from \code{\link{permutation_test}}).
Used upstream with \code{skeletion_build()} to obtain \code{skeleton}; it is
part of the API for a consistent joint-mapping workflow but is not read inside
\code{joint_map()}.}
}
\value{
A list containing pvalue and LOD for each term in the formula:
  \itemize{
    \item \code{pvalue}: P-value for each term
    \item \code{lod}: LOD score for each term
  }
}
\description{
Performs comprehensive joint mapping analysis for multiple traits using either Association Mapping (AM) 
or Linkage Mapping (LM) methods to identify QTL affecting multiple traits simultaneously. This function
implements sophisticated statistical methods for multivariate QTL analysis with comprehensive
cofactor selection and significance testing.
}
\examples{
\donttest{
# Example phenotype data
set.seed(1)
pheno_data <- data.frame(
  Trait1 = rnorm(100, mean = 100, sd = 15),
  Trait2 = rnorm(100, mean = 50, sd = 8),
  Popu = rep(c("Pop1", "Pop2"), each = 50)
)

# Example genotype data
geno_data <- matrix(sample(c(0,1,2), 100*50, replace = TRUE),
                    nrow = 100, ncol = 50)
colnames(geno_data) <- paste0("M", 1:50)

data1 <- cbind(pheno_data,geno_data) 

data1$Popu <- as.factor(data1$Popu)

terms <- c("Popu", paste0(colnames(geno_data),":Popu"))
formula1 <- reformulate(terms, response = "Trait1")

cut_off_list <- permutation_test(formula1, data1, n=10, alpha = 0.1, include="Popu")

skeleton <- skeletion_build(
  formula1, data1, strategy = "bidirection", metric = "SBC",
  cut_off_list = cut_off_list, include="Popu"
)

results <- joint_map(formula1, data1, skeleton, include = "Popu", cut_off_list = cut_off_list)

print(results)


}
}
\references{
Jiang, C. and Zeng, Z.B. (1995). Multiple trait analysis of genetic mapping for 
quantitative trait loci. Genetics, 140(3), 1111-1127.
}
\seealso{
\code{\link{permutation_test}} for permutation thresholds,
  \code{\link{genotype_prob}} for genotype probability calculations
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/skeleton_build.R
\name{skeletion_build}
\alias{skeletion_build}
\alias{skeleton_build}
\title{Fit the stepwise “skeleton” model for joint mapping}
\usage{
skeletion_build(
  formula,
  data,
  type = "linear",
  strategy = "bidirection",
  metric = "SL",
  include = NULL,
  cut_off_list
)

skeleton_build(
  formula,
  data,
  type = "linear",
  strategy = "bidirection",
  metric = "SL",
  include = NULL,
  cut_off_list
)
}
\arguments{
\item{formula}{A model formula (same as used in \code{\link{permutation_test}}).}

\item{data}{A data frame containing all variables in \code{formula}.}

\item{type}{Model type passed to \code{StepReg::stepwise()}. Default is
\code{"linear"}.}

\item{strategy}{Stepwise strategy, for example \code{"forward"},
\code{"backward"}, or \code{"bidirection"}. Default is \code{"bidirection"}.}

\item{metric}{Information criterion or selection metric in StepReg, for
example \code{"SL"}. Default is \code{"SL"}.}

\item{include}{Optional character vector of terms to keep in the stepwise
search, passed to \code{StepReg::stepwise()}. Default is \code{NULL}.}

\item{cut_off_list}{A list with a component \code{cut_off} that includes a
named entry \code{"pvalue"} (for example, the return value of
\code{\link{permutation_test}}). This value is used for both \code{sle} and
\code{sls} in the stepwise call.}
}
\value{
The best model object for the chosen \code{strategy} and
  \code{metric} (an element of the \code{StepReg::stepwise()} result,
  typically with a \code{call} and \code{call$formula}).
}
\description{
Runs \code{StepReg::stepwise()} with entry/stay levels set from a permutation
p-value threshold, and returns the selected best linear model. The result is
typically passed to \code{\link{joint_map}()} as the \code{skeleton} argument.
}
\seealso{
\code{\link{joint_map}}, \code{\link{permutation_test}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genotype_prob.R
\name{genotype_prob}
\alias{genotype_prob}
\title{Calculate Genotype Probabilities and Impute Missing Data}
\usage{
genotype_prob(GeneticMap, GenoData, method, croType = NULL, steps = 0, Gn = 2)
}
\arguments{
\item{GeneticMap}{A data frame containing genetic map information with columns:
\itemize{
  \item \code{marker}: Marker names (character)
  \item \code{chr}: Chromosome numbers (numeric or factor)
  \item \code{pos}: Genetic positions in centimorgans (numeric)
}}

\item{GenoData}{A matrix or data frame containing genotype data with individuals as rows 
and markers as columns. Genotype codes:
\itemize{
  \item \code{0}: Homozygous for reference allele
  \item \code{1}: Heterozygous
  \item \code{2}: Homozygous for alternative allele
  \item \code{9} or \code{NA}: Missing data
}}

\item{method}{Character string specifying the mapping method:
\itemize{
  \item \code{"AM"}: Association Mapping - combines map with genotype data
  \item \code{"LM"}: Linkage Mapping - performs missing data imputation
}}

\item{croType}{Character string specifying the cross type for LM method. Required when \code{method="LM"}:
\itemize{
  \item \code{"RIL"}: Recombinant Inbred Lines
  \item \code{"DH"}: Doubled Haploids
  \item \code{"F2"}: F2 population
  \item \code{"BCP1"}: Backcross to Parent 1
  \item \code{"BCP2"}: Backcross to Parent 2
  \item \code{"Fn"}: Advanced generation (n > 2)
}}

\item{steps}{Numeric value specifying the step size (in cM) for virtual marker creation in LM.
If 0, no virtual markers are created. Default is 0.}

\item{Gn}{Numeric value specifying the generation number for advanced populations.
Must be greater than 1. Default is 2.}
}
\value{
A data frame containing the processed genotype data with genetic map information
  and calculated genotype probabilities. The structure depends on the method:
  \itemize{
    \item \strong{AM method}: Original data combined with genetic map
    \item \strong{LM method}: Imputed genotype data with probabilities and optional virtual markers
  }
}
\description{
Calculates genotype probabilities and imputes missing genotype data for genetic mapping studies.
Supports both Association Mapping (AM) and Linkage Mapping (LM) methods with comprehensive
missing data handling and virtual marker creation capabilities.
}
\details{
This function provides comprehensive genotype data processing for genetic mapping:

\strong{Association Mapping (AM):}
\itemize{
  \item Simply combines genetic map with genotype data
  \item Converts missing values to code 9
  \item No imputation performed
}

\strong{Linkage Mapping (LM):}
\itemize{
  \item Validates genotype codes based on cross type expectations
  \item Imputes missing genotypes using flanking marker information
  \item Creates virtual markers at specified intervals when \code{steps > 0}
  \item Uses Haldane mapping function for recombination calculations
  \item Applies cross-type specific genotype probability calculations
}

\strong{Cross Type Genotype Expectations:}
\itemize{
  \item \code{RIL/DH}: Only codes 0 and 2 allowed
  \item \code{F2}: Codes 0, 1, and 2 allowed
  \item \code{BCP1/BCP2}: Codes 0, 1, and 2 with specific constraints
  \item \code{Fn}: Codes 0, 1, and 2 for advanced generations
}

\strong{Virtual Marker Creation:}
When \code{steps > 0}, virtual markers are created at regular intervals between
existing markers to improve mapping resolution and handle large gaps in the genetic map.
}
\examples{
# Example genetic map
genetic_map <- data.frame(
  marker = c("M1", "M2", "M3", "M4", "M5"),
  chr = c(1, 1, 1, 1, 1),
  pos = c(0, 10, 25, 40, 50)
)

# Example genotype data with missing values
geno_data <- matrix(
  c(2, 0, NA, 1, 2,
    1, 0, 2, NA, 0,
    2, 1, 0, 2, 1,
    0, NA, 1, 0, 2),
  nrow = 4, ncol = 5,
  dimnames = list(c("Ind1", "Ind2", "Ind3", "Ind4"), 
                  c("M1", "M2", "M3", "M4", "M5"))
)

# Association mapping (no imputation)
result_am <- genotype_prob(genetic_map, geno_data, method = "AM")
result_am

# Linkage mapping with virtual marker creation
result_lm_vm <- genotype_prob(genetic_map, geno_data, method = "LM", 
                           croType = "F2", steps = 5)
result_lm_vm

}
\references{
Haldane, J.B.S. (1919). The combination of linkage values and the calculation of 
distances between the loci of linked factors. Journal of Genetics, 8(3), 299-309.
}
\seealso{
\code{\link{haldane_map}} for recombination fraction calculations,
         \code{\link{expected_genotype_dist}} for expected genotype probabilities
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/expected_genotype_dist.R
\name{expected_genotype_dist}
\alias{expected_genotype_dist}
\alias{calculate_expected_genotype_distribution}
\title{Expected Allele Genotype Distribution Calculator}
\usage{
expected_genotype_dist(marType, croType, Gn = 2, x, y = 0)
}
\arguments{
\item{marType}{Character string specifying the marker type. Options include:
\itemize{
  \item \code{"22", "21", "20", "12", "11", "10", "02", "01", "00"}: Both flanking markers observed
  \item \code{"N2", "N1", "N0"}: Only right flanking marker observed
  \item \code{"2N", "1N", "0N"}: Only left flanking marker observed
}
These codes represent different combinations of flanking marker genotypes.}

\item{croType}{Character string specifying the cross type:
\itemize{
  \item \code{"Fn"}: F-n generation populations
  \item \code{"BCP1"}: Backcross to parent 1
  \item \code{"BCP2"}: Backcross to parent 2
  \item \code{"F2"}: F2 generation
  \item \code{"DH"}: Doubled haploid
  \item \code{"RIL"}: Recombinant inbred line
}}

\item{Gn}{Numeric value specifying the generation number. Must be greater than 0.}

\item{x}{Numeric value representing recombination fraction between loci A and Q.
Must be between 0 and 0.5.}

\item{y}{Numeric value representing recombination fraction between loci Q and B.
Must be between 0 and 0.5. Default is 0.}
}
\value{
A numeric value representing the expected probability of the specified genotype.
  The result is typically between -1 and 1, where positive values indicate higher
  probability of the target genotype.
}
\description{
Calculates expected allele genotype distribution probabilities for different marker types
and cross types in genetic mapping studies. This function implements sophisticated algorithms
to compute genotype probabilities based on flanking marker information and population structure.
}
\details{
This function calculates expected genotype probabilities using different approaches:

\strong{Marker Type Classification:}
\itemize{
  \item \strong{Double flanking markers} (\code{"22", "21", "20", etc.}): Both left and right
        flanking markers are observed, providing maximum information
  \item \strong{Single right flanking marker} (\code{"N2", "N1", "N0"}): Only the right
        flanking marker is observed
  \item \strong{Single left flanking marker} (\code{"2N", "1N", "0N"}): Only the left
        flanking marker is observed
}

\strong{Calculation Methods by Cross Type:}
\itemize{
  \item \strong{Fn populations}: Uses \code{\link{genotype_freq}} for complex calculations
        involving multiple genotype classes
  \item \strong{F2, DH, RIL}: Uses analytical formulas based on classical genetics
  \item \strong{BCP1, BCP2}: Uses \code{\link{genotype_freq}} with specific genotype indices
}

\strong{Mathematical Framework:}
The function uses different probability calculations depending on the available information:
\itemize{
  \item When both flanking markers are observed, it uses conditional probabilities
  \item When only one flanking marker is observed, it uses marginal probabilities
  \item The calculations incorporate recombination fractions and population-specific parameters
}

\strong{Genotype Coding:}
The marker types use a coding system where:
\itemize{
  \item \code{2}: Homozygous for alternative allele
  \item \code{1}: Heterozygous
  \item \code{0}: Homozygous for reference allele
  \item \code{N}: Missing or unknown genotype
}
}
\examples{
# Calculate probability for RIL population
prob_ril <- expected_genotype_dist("00", "RIL", Gn = 2, x = 0.1, y = 0.2)

# Example with different recombination fractions
prob_low_rec <- expected_genotype_dist("22", "F2", Gn = 2, x = 0.05, y = 0.05)
prob_high_rec <- expected_genotype_dist("22", "F2", Gn = 2, x = 0.3, y = 0.4)

}
\references{
Haldane, J.B.S. (1919). The combination of linkage values and the calculation of 
distances between the loci of linked factors. Journal of Genetics, 8(3), 299-309.
}
\seealso{
\code{\link{genotype_freq}} for genotype frequency calculations,
         \code{\link{genotype_prob}} for missing genotype imputation
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/permutation_test.R
\name{permutation_test}
\alias{permutation_test}
\title{Permutation Test for Stepwise Regression}
\usage{
permutation_test(
  formula,
  data,
  n = 1000,
  alpha = 0.1,
  include = NULL,
  strategy = "bidirection",
  metric = "SBC",
  type = "linear",
  verbose = FALSE
)
}
\arguments{
\item{formula}{A model formula defining the response and candidate predictors for
permutation testing.}

\item{data}{A data frame containing all variables referenced in \code{formula},
including response trait(s) and predictor variables.}

\item{n}{Integer number of permutations to run. Larger values provide more stable
empirical thresholds but increase computation time. Default is \code{1000}.}

\item{alpha}{Numeric significance level used to extract empirical cutoff values
from the permutation distributions. Must be between 0 and 1. Default is \code{0.1}.}

\item{include}{Optional character vector of variable names that should always be
included during stepwise model selection. If \code{NULL}, inclusion terms can be
inferred from interaction terms in \code{formula}. Default is \code{NULL}.}

\item{strategy}{Stepwise selection strategy passed to \code{StepReg::stepwise()}.
Typical values include \code{"forward"}, \code{"backward"}, and \code{"bidirection"}.
Default is \code{"bidirection"}.}

\item{metric}{Model selection metric used inside stepwise regression.
Typical values include \code{"AIC"}, \code{"BIC"}, or \code{"SBC"}.
Default is \code{"SBC"}.}

\item{type}{Type of regression model.
Typical values include \code{"linear"}, \code{"logistic"}, and \code{"poisson"}.
Default is \code{"linear"}.}

\item{verbose}{Logical; if \code{TRUE}, emit progress \code{message()}s during
permutations. Default is \code{FALSE}.}
}
\value{
A list containing the empirical significance thresholds for p-value and LOD score with:
  \itemize{
    \item \strong{cut_off}: A vector containing the empirical significance thresholds for p-value and LOD score
    \item \strong{pvalue}: A vector containing the p-values for each permutation
    \item \strong{lod}: A vector containing the LOD scores for each permutation
  }
}
\description{
Performs permutation tests for stepwise regression to determine empirical significance thresholds
for QTL detection using stepwise regression. This function implements a comprehensive permutation
testing framework for stepwise regression.
}
\details{
This function implements a comprehensive permutation testing framework.

\strong{Output Interpretation:}
\itemize{
  \item \strong{P-values}: Empirical significance thresholds for p-value
  \item \strong{LOD scores}: Empirical significance thresholds for LOD score
}
}
\examples{
\donttest{
# Example phenotype data
set.seed(1)
pheno_data <- data.frame(
  Trait1 = rnorm(100, mean = 100, sd = 15),
  Trait2 = rnorm(100, mean = 50, sd = 8),
  Popu = rep(c("Pop1", "Pop2"), each = 50)
)

# Example genotype data
geno_data <- matrix(sample(c(0,1,2), 100*50, replace = TRUE), 
                    nrow = 100, ncol = 50)
colnames(geno_data) <- paste0("M", 1:50)

data1 <- cbind(pheno_data,geno_data)

data1$Popu <- as.factor(data1$Popu)

terms <- c("Popu", paste0(colnames(geno_data), ":Popu"))
formula1 <- reformulate(terms, response = "Trait1")

cut_off_list <- permutation_test(formula1, data1, n = 10, alpha = 0.1)

formula2 <- reformulate(terms, response = "cbind(Trait1,Trait2)")

cut_off_list <- permutation_test(formula2, data1, n = 10, alpha = 0.1)
}

}

\name{GeneticMap}
\alias{GeneticMap}
\docType{data}
\title{
Data about genetic map
}
\description{
A data frame with 1722 observations on the following 3 variables, where the first column is marker name, the second column is chromosome numeric ID and the last column is marker genetic distance position
}
\usage{data("GeneticMap")}
\format{
  A data frame with 1722 observations on the following 3 variables.
  \describe{
    \item{\code{marker}}{a vector for marker name}
    \item{\code{chr}}{a numeric vector for chromosome id}
    \item{\code{pos}}{a numeric vector for marker genetic distance position}
  }
}
\details{
The name of marker must be the same with marker`s name of data in GenoData, and the chromosome ID and genetic distance position are numeric, otherwise an error will occur.
}
\references{
Hu, H., Meng, Y., Wang, H., Liu, H., and Chen, S. (2012). Identifying quantitative trait loci and determining closely related, stalk traits for rind penetrometer resistance in a high-oil maize, population. Theoretical and Applied Genetics, 124(8), 1439-1447.

Hu, H., Liu, W., Fu, Z., Homann, L., Technow, F., and Wang, H., et al. (2013). Qtl mapping of stalk bending strength in a recombinant inbred line maize population. Theoretical and Applied Genetics, 126(9), 2257-66.
}
\examples{
data(GeneticMap)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/haldane_map.R
\name{haldane_map}
\alias{haldane_map}
\alias{haldane_mapping_function}
\title{Haldane Mapping Function}
\usage{
haldane_map(x)
}
\arguments{
\item{x}{A numeric value representing the genetic distance in Morgan units (centimorgans/100).
Must be non-negative.}
}
\value{
A numeric value representing the recombination fraction (r) between 0 and 0.5.
}
\description{
Converts genetic distance in Morgan units to recombination fraction using the Haldane mapping function.
This function implements the Haldane mapping function which assumes no interference between crossovers.
}
\details{
The Haldane mapping function is defined as:
\deqn{r = \frac{1}{2}(1 - e^{-2x})}
where \eqn{x} is the genetic distance in Morgan units and \eqn{r} is the recombination fraction.

This function assumes no interference between crossovers, meaning that the occurrence
of one crossover does not affect the probability of other crossovers occurring nearby.
}
\examples{
# Convert 0.1 Morgan (10 cM) to recombination fraction
haldane_map(0.1)

# Convert 0.5 Morgan (50 cM) to recombination fraction  
haldane_map(0.5)

# Convert 1.0 Morgan (100 cM) to recombination fraction
haldane_map(1.0)

}
\references{
Haldane, J.B.S. (1919). The combination of linkage values and the calculation of 
distances between the loci of linked factors. Journal of Genetics, 8(3), 299-309.
}
\seealso{
\code{\link{genotype_prob}} for genotype probability calculations
}

\name{JM4QTN-package}
\alias{JM4QTN-package}
\alias{JM4QTN}
\docType{package}
\title{
Joint Mapping for QTN
}
\description{
The "JM4QTN" package is based on composite interval mapping and regression analysis to do association analysis and linkage analysis for multiple-line cross population and multiple traits. The package can also identify pleiotropic or linked QTL.
}
\details{
\tabular{ll}{
Package: \tab JM4QTN\cr
Type: \tab Package\cr
Version: \tab 1.0\cr
Date: \tab 2017-03-23\cr
License: \tab GPL (>= 2)\cr
}
}
\author{
JunhuLi,GuoliangLi,HeliChen,KunCheng,WenxinLiu

Maintainer: JunhuiLi<junhuili@cau.edu.cn>
}
\references{
Junhui Li, Haixiao Hu, Yujie Meng, Kun Cheng, Guoliang Li, Wenxin Liu, and Shaojiang Chen.(2016)Pleiotropic QTL detection for stalk traits in maize and related R package programming. Journal of China Agricultural University. DOI 10.11841/j.issn.1007-4333.2016.06.00(in chinese)
}
\keyword{ package JointMapping QTN}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_stats.R
\name{pheno_stats}
\alias{pheno_stats}
\title{Statistical Analysis of Phenotype Data}
\usage{
pheno_stats(phenoData, defineForm = NULL, effNotation = "G")
}
\arguments{
\item{phenoData}{A data frame containing phenotype data with at least 5 columns.
The first 4 columns must be: Environment (E), Block (B), Repetition (R), and Genotype (G).
Additional columns contain trait measurements for statistical analysis.}

\item{defineForm}{Optional character vector of custom formula strings for statistical analysis.
If NULL, default formulas are automatically generated based on the data structure.}

\item{effNotation}{Character string specifying the effect notation for least squares means.
Default is "G" for genotype effect.}
}
\value{
A list containing comprehensive statistical analysis results for each trait:
  \item{normality_test}{Results of Shapiro-Wilk normality test}
  \item{formula}{Formula used for the statistical model}
  \item{ANOVA}{Complete analysis of variance results}
  \item{lsmeans}{Least squares means for genotypes with standard errors}
}
\description{
Performs comprehensive statistical analysis on phenotype data including normality tests, 
analysis of variance (ANOVA), and least squares means calculations for genetic studies.
}
\details{
This function performs a comprehensive statistical analysis pipeline:

\enumerate{
  \item \strong{Normality Test}: Uses Shapiro-Wilk test to assess normality of each trait
  \item \strong{Model Selection}: Automatically generates appropriate statistical models based on data structure
  \item \strong{ANOVA}: Performs analysis of variance to test significance of effects
  \item \strong{Least Squares Means}: Calculates adjusted means for genotypes
}

\strong{Automatic Model Generation:}
The function automatically generates appropriate formulas based on the experimental design:
\itemize{
  \item Multiple environments and blocks: \code{trait ~ E + G + E:G + B\%in\%E}
  \item Multiple environments only: \code{trait ~ E*G}
  \item Custom formulas: User-defined formulas when \code{defineForm} is provided
}

\strong{Data Requirements:}
The phenotype data must have the following structure:
\itemize{
  \item Column 1: Environment (E) - factor variable
  \item Column 2: Block (B) - factor variable  
  \item Column 3: Repetition (R) - factor variable
  \item Column 4: Genotype (G) - factor variable
  \item Columns 5+: Trait measurements - numeric variables
}
}
\examples{

# Example with multiple environments and blocks
pheno_data <- data.frame(
  E = rep(c("Env1", "Env2", "Env3"), each = 60),
  B = rep(c("Block1", "Block2"), each = 30, times = 3),
  R = rep(1:5, 36),
  G = rep(1:12, 15),
  Height = rnorm(180, 175, 8),
  Weight = rnorm(180, 75, 12)
)

# Perform statistical analysis with default formulas
results <- pheno_stats(pheno_data)

# View normality test results
results$Height$normality_test

# View ANOVA results
results$Height$ANOVA

# View least squares means
results$Height$lsmeans

# Example with custom formulas
custom_formulas <- c(
  "Height ~ E + G + E:G + B\%in\%E",
  "Weight ~ E + G + E:G"
)
results_custom <- pheno_stats(pheno_data, defineForm = custom_formulas)

}
\references{
Shapiro, S.S. and Wilk, M.B. (1965). An analysis of variance test for normality. 
Biometrika, 52(3-4), 591-611.
}
\seealso{
\code{\link{joint_map}} for joint mapping analysis
}

\name{GenoData}
\alias{GenoData}
\docType{data}
\title{
Data about molecular markers genotype
}
\description{
A data frame with 647 individual observations on the following 1722 variables, where 2, 1 and 0 stands for different markers genotype AA AB and BB, NA for missing marker genotype
}
\usage{data("GenoData")}
\format{
  A data frame with 647 individual observations on the following 1722 marker genotype variables
}
\details{
Missing markers are subsitituted only by NA, otherwise an error will occur.
}
\references{
Hu, H., Meng, Y., Wang, H., Liu, H., and Chen, S. (2012). Identifying quantitative trait loci and determining closely related, stalk traits for rind penetrometer resistance in a high-oil maize, population. Theoretical and Applied Genetics, 124(8), 1439-1447.

Hu, H., Liu, W., Fu, Z., Homann, L., Technow, F., and Wang, H., et al. (2013). Qtl mapping of stalk bending strength in a recombinant inbred line maize population. Theoretical and Applied Genetics, 126(9), 2257-66.
}
\examples{
data(GenoData)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genotype_freq.R
\name{genotype_freq}
\alias{genotype_freq}
\alias{calculate_genotype_frequencies}
\title{Genotype Frequency Calculator for Genetic Populations}
\usage{
genotype_freq(
  cross_type,
  generation = 2,
  genotype_index,
  recomb_aq,
  recomb_qb = 0
)
}
\arguments{
\item{cross_type}{Population type used in the calculation:
\itemize{
  \item \code{"Fn"}: F-n generation populations (n > 2)
  \item \code{"BCP1"}: Backcross to parent 1 populations
  \item \code{"BCP2"}: Backcross to parent 2 populations
  \item \code{"F2"}: F2 generation populations
  \item \code{"DH"}: Doubled haploid populations
  \item \code{"RIL"}: Recombinant inbred line populations
}}

\item{generation}{Generation number (for example, 2 for F2, 3 for F3). Must be greater than 1.}

\item{genotype_index}{Index of the genotype class to evaluate.
Valid range depends on `cross_type`:
\itemize{
  \item \code{"Fn"}: 1-20 (20 different genotype classes)
  \item \code{"BCP1/BCP2"}: 1-8 (8 genotype classes)
  \item \code{"F2"}: 1-9 (9 genotype classes)
}}

\item{recomb_aq}{Recombination fraction between loci A and Q.
Use a value between 0 and 0.5.}

\item{recomb_qb}{Recombination fraction between loci Q and B.
Use a value between 0 and 0.5. Default is 0.}
}
\value{
A numeric value representing the frequency of the specified genotype in the given generation.
  The result is always between 0 and 1.
}
\description{
Calculates genotype frequencies for different cross types and generations in genetic mapping studies.
This function implements complex recurrence relations to compute genotype frequencies in various
genetic populations including Fn, backcross, and advanced generation populations.
}
\details{
This function calculates genotype frequencies using sophisticated mathematical models:

\strong{For Fn Populations (y > 0):}
\itemize{
  \item Handles 20 different genotype classes with complex recurrence relations
  \item Accounts for recombination between three loci (A, Q, B)
  \item Uses the relationship \eqn{z = x + y - 2xy} for flanking marker recombination
  \item Implements generation-by-generation frequency calculations
}

\strong{For BCP1 Populations:}
\itemize{
  \item Handles 8 genotype classes
  \item Backcross to the first parent (AAQQBB)
  \item Specific recurrence relations for each genotype class
}

\strong{For BCP2 Populations:}
\itemize{
  \item Handles 8 genotype classes
  \item Backcross to the second parent (aaqqbb)
  \item Different initial conditions and recurrence relations
}

\strong{For F2 Populations (y = 0):}
\itemize{
  \item Handles 9 genotype classes
  \item Simplified two-locus model
  \item Standard F2 generation frequencies
}

\strong{Mathematical Foundation:}
The function uses recurrence relations to calculate genotype frequencies across generations.
For each generation, the frequency of each genotype class is computed based on the
frequencies in the previous generation and the recombination fractions between loci.
}
\examples{
# Calculate frequency for F2 generation, genotype 1, with recombination fraction 0.1
freq_f2 <- genotype_freq("Fn", generation = 2, genotype_index = 1, recomb_aq = 0.1, recomb_qb = 0)

# Calculate frequency for BCP1 generation, genotype 5, with recombination fractions
freq_bcp1 <- genotype_freq(
  "BCP1", generation = 3, genotype_index = 5, recomb_aq = 0.15, recomb_qb = 0.25
)

# Calculate frequency for Fn generation, genotype 10, with recombination fractions
freq_fn <- genotype_freq(
  "Fn", generation = 4, genotype_index = 10, recomb_aq = 0.2, recomb_qb = 0.3
)

# Calculate frequency for BCP2 generation, genotype 3
freq_bcp2 <- genotype_freq(
  "BCP2", generation = 2, genotype_index = 3, recomb_aq = 0.1, recomb_qb = 0.2
)

# Example with different generation numbers
freq_gen3 <- genotype_freq("Fn", generation = 3, genotype_index = 1, recomb_aq = 0.1, recomb_qb = 0)
freq_gen5 <- genotype_freq("Fn", generation = 5, genotype_index = 1, recomb_aq = 0.1, recomb_qb = 0)

}
\references{
Haldane, J.B.S. (1919). The combination of linkage values and the calculation of 
distances between the loci of linked factors. Journal of Genetics, 8(3), 299-309.
}
\seealso{
\code{\link{expected_genotype_dist}} for expected allele genotype distribution calculations
}

\name{PhenoData}
\alias{PhenoData}
\docType{data}
\title{
Data about all phenotype
}
\description{
A data frame with 647 observations on the following 10 variables, the phenotype value is calculated by best linear unbias estimates.
}
\usage{data("PhenoData")}
\format{
  A data frame with 647 observations on the following 13 variables.
  \describe{
    \item{\code{Indi}}{The ID of individuals, which is a factor with levels of Pop1 with 131 individuals, Pop2 with 120 individuals, Pop3 with 200 individuals and Pop4 with 200 individuals}
    \item{\code{Popu}}{a factor with levels \code{Pop1} \code{Pop2} \code{Pop3} \code{Pop4} for multiple-line cross population}
    \item{\code{Ma}}{a factor with levels \code{A} \code{D} for parent 1}
    \item{\code{Pa}}{a factor with levels \code{B} \code{C} \code{E} \code{F} for parent 2}
    \item{\code{newEC1}}{a numeric vector for trait newEC1}
    \item{\code{newEC2}}{a numeric vector for trait newEC2}
    \item{\code{newEC3}}{a numeric vector for trait newEC3}
    \item{\code{BM1}}{a numeric vector for trait BM1}
    \item{\code{BM2}}{a numeric vector for trait BM2}
    \item{\code{BM3}}{a numeric vector for trait BM3}
	\item{\code{predPH1}}{a numeric vector for trait predPH1}
    \item{\code{predPH2}}{a numeric vector for trait predPH2}
    \item{\code{predPH3}}{a numeric vector for trait predPH3}
  }
}
\details{
Popu is the levels of multiple cross populations with different parent crosses, if it is filled with several levels, then linkage mapping or association mapping analysis is done for multiple cross populations, otherwise just for only one population
}
\references{
Hu, H., Meng, Y., Wang, H., Liu, H., and Chen, S. (2012). Identifying quantitative trait loci and determining closely related, stalk traits for rind penetrometer resistance in a high-oil maize, population. Theoretical and Applied Genetics, 124(8), 1439-1447.

Hu, H., Liu, W., Fu, Z., Homann, L., Technow, F., and Wang, H., et al. (2013). Qtl mapping of stalk bending strength in a recombinant inbred line maize population. Theoretical and Applied Genetics, 126(9), 2257-66.
}
\examples{
data(PhenoData)
}
\keyword{datasets}

\name{GenoData_S2}
\alias{GenoData_S2}
\docType{data}
\title{
Data about information of molecular markers chromosome id, genetic distance position and estimated missing markers genotype with steps = 2
}
\description{
A data frame with 647 individual observations, chromosome id and genetic distance position on the following 2431 molecular maker variables
}
\usage{data("GenoData_S2")}
\format{
  A data frame with 649 observations on the following 2431 variables
  \describe{
    A data frame with 649 observations on the following 2431 variables, the rows 'chr' and 'pos' is the chromosome id and genetic distance position of markers, other rows are the marker genotype on 647 individuals, where 0 and 2 stands for genotype AA and BB respectivly, decimal is marker conditional probability genotype with steps = 2.
  }
}
\details{
This data frame is combined from geneticMap and genotype data with estimated missing markers, which can be calculated by function calGenoProb(GeneticMap, GenoData, steps=2, croType, Gn)
}
\references{
Hu, H., Meng, Y., Wang, H., Liu, H., and Chen, S. (2012). Identifying quantitative trait loci and determining closely related, stalk traits for rind penetrometer resistance in a high-oil maize, population. Theoretical and Applied Genetics, 124(8), 1439-1447.

Hu, H., Liu, W., Fu, Z., Homann, L., Technow, F., and Wang, H., et al. (2013). Qtl mapping of stalk bending strength in a recombinant inbred line maize population. Theoretical and Applied Genetics, 126(9), 2257-66.
}
\examples{
data(GenoData_S2)
}
\keyword{datasets}
